open AffTerm
open BinNums
open CstrC
open Datatypes
open LinTerm
open NumC
open ProgVar

type cmpT =
| Eq
| Le
| Lt
| Neq

type binl =
| AND
| OR

module ZCond = 
 struct 
  module Term = ZTerm
  
  (** val cmpDenote_dec : cmpT -> ZNum.t -> ZNum.t -> bool **)
  
  let cmpDenote_dec cmp n1 n2 =
    match cmp with
    | Eq -> ZNum.eqDec n1 n2
    | Le -> if ZNum.ltLeDec n2 n1 then false else true
    | Lt -> ZNum.ltLeDec n1 n2
    | Neq -> if ZNum.eqDec n1 n2 then false else true
  
  type cond =
  | Basic of bool
  | Atom of cmpT * ZTerm.term * ZTerm.term
  | BinL of binl * cond * cond
  | Not of cond
  
  (** val cond_rect :
      (bool -> 'a1) -> (cmpT -> ZTerm.term -> ZTerm.term -> 'a1) -> (binl ->
      cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond ->
      'a1 **)
  
  let rec cond_rect f f0 f1 f2 = function
  | Basic b -> f b
  | Atom (oc, tl, tr) -> f0 oc tl tr
  | BinL (op, cl, cr) ->
    f1 op cl (cond_rect f f0 f1 f2 cl) cr (cond_rect f f0 f1 f2 cr)
  | Not c0 -> f2 c0 (cond_rect f f0 f1 f2 c0)
  
  (** val cond_rec :
      (bool -> 'a1) -> (cmpT -> ZTerm.term -> ZTerm.term -> 'a1) -> (binl ->
      cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond ->
      'a1 **)
  
  let rec cond_rec f f0 f1 f2 = function
  | Basic b -> f b
  | Atom (oc, tl, tr) -> f0 oc tl tr
  | BinL (op, cl, cr) ->
    f1 op cl (cond_rec f f0 f1 f2 cl) cr (cond_rec f f0 f1 f2 cr)
  | Not c0 -> f2 c0 (cond_rec f f0 f1 f2 c0)
  
  type t = cond
  
  (** val sat_dec : t -> ZNum.t Mem.t -> bool **)
  
  let rec sat_dec c m =
    match c with
    | Basic b -> if b then true else false
    | Atom (oc, tl, tr) ->
      cmpDenote_dec oc (ZTerm.eval tl m) (ZTerm.eval tr m)
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> if sat_dec cl m then sat_dec cr m else false
       | OR -> if sat_dec cl m then true else sat_dec cr m)
    | Not c0 -> if sat_dec c0 m then false else true
  
  (** val xsat_dec : t -> ZNum.t Mem.t -> ZNum.t Mem.t -> bool **)
  
  let rec xsat_dec c old new0 =
    match c with
    | Basic b -> if b then true else false
    | Atom (oc, tl, tr) ->
      cmpDenote_dec oc (ZTerm.xeval tl old new0) (ZTerm.xeval tr old new0)
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> if xsat_dec cl old new0 then xsat_dec cr old new0 else false
       | OR -> if xsat_dec cl old new0 then true else xsat_dec cr old new0)
    | Not c0 -> if xsat_dec c0 old new0 then false else true
  
  (** val mdBound : cond -> positive -> positive **)
  
  let rec mdBound c bound =
    match c with
    | Basic b -> bound
    | Atom (oc, tl, tr) -> ZTerm.mdBound tl (ZTerm.mdBound tr bound)
    | BinL (op, cl, cr) -> mdBound cl (mdBound cr bound)
    | Not c0 -> mdBound c0 bound
  
  (** val map : cond -> PVar.t Mem.t -> cond **)
  
  let rec map c f =
    match c with
    | Basic b -> c
    | Atom (oc, tl, tr) -> Atom (oc, (ZTerm.map tl f), (ZTerm.map tr f))
    | BinL (op, cl, cr) -> BinL (op, (map cl f), (map cr f))
    | Not c0 -> Not (map c0 f)
  
  (** val xmap : cond -> PVar.t Mem.t -> PVar.t Mem.t -> cond **)
  
  let rec xmap c old new0 =
    match c with
    | Basic b -> c
    | Atom (oc, tl, tr) ->
      Atom (oc, (ZTerm.xmap tl old new0), (ZTerm.xmap tr old new0))
    | BinL (op, cl, cr) -> BinL (op, (xmap cl old new0), (xmap cr old new0))
    | Not c0 -> Not (xmap c0 old new0)
  
  (** val alt :
      (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) ->
      cmpT -> ZTerm.term -> ZTerm.term -> t **)
  
  let alt altLt0 altNeq cmp t1 t2 =
    match cmp with
    | Lt -> altLt0 t1 t2
    | Neq -> altNeq t1 t2
    | _ -> Atom (cmp, t1, t2)
  
  (** val dual :
      (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) ->
      cmpT -> ZTerm.term -> ZTerm.term -> t **)
  
  let dual altLt0 altNeq cmp t1 t2 =
    match cmp with
    | Eq -> altNeq t1 t2
    | Le -> altLt0 t2 t1
    | Lt -> Atom (Le, t2, t1)
    | Neq -> Atom (Eq, t1, t2)
  
  (** val nnf :
      (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) -> t
      -> t **)
  
  let nnf altLt0 altNeq =
    let rec nnf0 c = match c with
    | Basic b -> c
    | Atom (op, tl, tr) ->
      (match op with
       | Lt -> altLt0 tl tr
       | Neq -> altNeq tl tr
       | _ -> Atom (op, tl, tr))
    | BinL (op, cl, cr) -> BinL (op, (nnf0 cl), (nnf0 cr))
    | Not c0 -> nnfNot0 c0
    and nnfNot0 = function
    | Basic b -> Basic (negb b)
    | Atom (op, tl, tr) ->
      (match op with
       | Eq -> altNeq tl tr
       | Le -> altLt0 tr tl
       | Lt -> Atom (Le, tr, tl)
       | Neq -> Atom (Eq, tl, tr))
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> BinL (OR, (nnfNot0 cl), (nnfNot0 cr))
       | OR -> BinL (AND, (nnfNot0 cl), (nnfNot0 cr)))
    | Not c0 -> nnf0 c0
    in nnf0
  
  (** val nnfNot :
      (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) -> t
      -> t **)
  
  let nnfNot altLt0 altNeq =
    let rec nnf0 c = match c with
    | Basic b -> c
    | Atom (op, tl, tr) ->
      (match op with
       | Lt -> altLt0 tl tr
       | Neq -> altNeq tl tr
       | _ -> Atom (op, tl, tr))
    | BinL (op, cl, cr) -> BinL (op, (nnf0 cl), (nnf0 cr))
    | Not c0 -> nnfNot0 c0
    and nnfNot0 = function
    | Basic b -> Basic (negb b)
    | Atom (op, tl, tr) ->
      (match op with
       | Eq -> altNeq tl tr
       | Le -> altLt0 tr tl
       | Lt -> Atom (Le, tr, tl)
       | Neq -> Atom (Eq, tl, tr))
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> BinL (OR, (nnfNot0 cl), (nnfNot0 cr))
       | OR -> BinL (AND, (nnfNot0 cl), (nnfNot0 cr)))
    | Not c0 -> nnf0 c0
    in nnfNot0
  
  (** val altNeqByOR :
      (ZTerm.term -> ZTerm.term -> t) -> ZTerm.term -> ZTerm.term -> cond **)
  
  let altNeqByOR altLt0 t1 t2 =
    BinL (OR, (altLt0 t1 t2), (altLt0 t2 t1))
  
  (** val altLt : ZTerm.term -> ZTerm.term -> cond **)
  
  let altLt t1 t2 =
    Atom (Le, (ZTerm.Plus (t1, (ZTerm.Cte (Zpos Coq_xH)))), t2)
  
  (** val nnfForAssume : t -> t **)
  
  let nnfForAssume =
    nnf (fun t1 t2 -> Atom (Le, (ZTerm.Plus (t1, (ZTerm.Cte (Zpos Coq_xH)))),
      t2))
      (let altLt0 = fun t1 t2 -> Atom (Le, (ZTerm.Plus (t1, (ZTerm.Cte (Zpos
         Coq_xH)))), t2)
       in
       fun t1 t2 -> BinL (OR, (altLt0 t1 t2), (altLt0 t2 t1)))
  
  (** val nnfForRequire : t -> t **)
  
  let nnfForRequire =
    nnf (fun t1 t2 -> Atom (Le, (ZTerm.Plus (t1, (ZTerm.Cte (Zpos Coq_xH)))),
      t2)) (fun x x0 -> Atom (Neq, x, x0))
 end

module QCond = 
 struct 
  module Term = QTerm
  
  (** val cmpDenote_dec : cmpT -> QNum.t -> QNum.t -> bool **)
  
  let cmpDenote_dec cmp n1 n2 =
    match cmp with
    | Eq -> QNum.eqDec n1 n2
    | Le -> if QNum.ltLeDec n2 n1 then false else true
    | Lt -> QNum.ltLeDec n1 n2
    | Neq -> if QNum.eqDec n1 n2 then false else true
  
  type cond =
  | Basic of bool
  | Atom of cmpT * QTerm.term * QTerm.term
  | BinL of binl * cond * cond
  | Not of cond
  
  (** val cond_rect :
      (bool -> 'a1) -> (cmpT -> QTerm.term -> QTerm.term -> 'a1) -> (binl ->
      cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond ->
      'a1 **)
  
  let rec cond_rect f f0 f1 f2 = function
  | Basic b -> f b
  | Atom (oc, tl, tr) -> f0 oc tl tr
  | BinL (op, cl, cr) ->
    f1 op cl (cond_rect f f0 f1 f2 cl) cr (cond_rect f f0 f1 f2 cr)
  | Not c0 -> f2 c0 (cond_rect f f0 f1 f2 c0)
  
  (** val cond_rec :
      (bool -> 'a1) -> (cmpT -> QTerm.term -> QTerm.term -> 'a1) -> (binl ->
      cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond ->
      'a1 **)
  
  let rec cond_rec f f0 f1 f2 = function
  | Basic b -> f b
  | Atom (oc, tl, tr) -> f0 oc tl tr
  | BinL (op, cl, cr) ->
    f1 op cl (cond_rec f f0 f1 f2 cl) cr (cond_rec f f0 f1 f2 cr)
  | Not c0 -> f2 c0 (cond_rec f f0 f1 f2 c0)
  
  type t = cond
  
  (** val sat_dec : t -> QNum.t Mem.t -> bool **)
  
  let rec sat_dec c m =
    match c with
    | Basic b -> if b then true else false
    | Atom (oc, tl, tr) ->
      cmpDenote_dec oc (QTerm.eval tl m) (QTerm.eval tr m)
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> if sat_dec cl m then sat_dec cr m else false
       | OR -> if sat_dec cl m then true else sat_dec cr m)
    | Not c0 -> if sat_dec c0 m then false else true
  
  (** val xsat_dec : t -> QNum.t Mem.t -> QNum.t Mem.t -> bool **)
  
  let rec xsat_dec c old new0 =
    match c with
    | Basic b -> if b then true else false
    | Atom (oc, tl, tr) ->
      cmpDenote_dec oc (QTerm.xeval tl old new0) (QTerm.xeval tr old new0)
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> if xsat_dec cl old new0 then xsat_dec cr old new0 else false
       | OR -> if xsat_dec cl old new0 then true else xsat_dec cr old new0)
    | Not c0 -> if xsat_dec c0 old new0 then false else true
  
  (** val mdBound : cond -> positive -> positive **)
  
  let rec mdBound c bound =
    match c with
    | Basic b -> bound
    | Atom (oc, tl, tr) -> QTerm.mdBound tl (QTerm.mdBound tr bound)
    | BinL (op, cl, cr) -> mdBound cl (mdBound cr bound)
    | Not c0 -> mdBound c0 bound
  
  (** val map : cond -> PVar.t Mem.t -> cond **)
  
  let rec map c f =
    match c with
    | Basic b -> c
    | Atom (oc, tl, tr) -> Atom (oc, (QTerm.map tl f), (QTerm.map tr f))
    | BinL (op, cl, cr) -> BinL (op, (map cl f), (map cr f))
    | Not c0 -> Not (map c0 f)
  
  (** val xmap : cond -> PVar.t Mem.t -> PVar.t Mem.t -> cond **)
  
  let rec xmap c old new0 =
    match c with
    | Basic b -> c
    | Atom (oc, tl, tr) ->
      Atom (oc, (QTerm.xmap tl old new0), (QTerm.xmap tr old new0))
    | BinL (op, cl, cr) -> BinL (op, (xmap cl old new0), (xmap cr old new0))
    | Not c0 -> Not (xmap c0 old new0)
  
  (** val alt :
      (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) ->
      cmpT -> QTerm.term -> QTerm.term -> t **)
  
  let alt altLt0 altNeq cmp t1 t2 =
    match cmp with
    | Lt -> altLt0 t1 t2
    | Neq -> altNeq t1 t2
    | _ -> Atom (cmp, t1, t2)
  
  (** val dual :
      (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) ->
      cmpT -> QTerm.term -> QTerm.term -> t **)
  
  let dual altLt0 altNeq cmp t1 t2 =
    match cmp with
    | Eq -> altNeq t1 t2
    | Le -> altLt0 t2 t1
    | Lt -> Atom (Le, t2, t1)
    | Neq -> Atom (Eq, t1, t2)
  
  (** val nnf :
      (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) -> t
      -> t **)
  
  let nnf altLt0 altNeq =
    let rec nnf0 c = match c with
    | Basic b -> c
    | Atom (op, tl, tr) ->
      (match op with
       | Lt -> altLt0 tl tr
       | Neq -> altNeq tl tr
       | _ -> Atom (op, tl, tr))
    | BinL (op, cl, cr) -> BinL (op, (nnf0 cl), (nnf0 cr))
    | Not c0 -> nnfNot0 c0
    and nnfNot0 = function
    | Basic b -> Basic (negb b)
    | Atom (op, tl, tr) ->
      (match op with
       | Eq -> altNeq tl tr
       | Le -> altLt0 tr tl
       | Lt -> Atom (Le, tr, tl)
       | Neq -> Atom (Eq, tl, tr))
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> BinL (OR, (nnfNot0 cl), (nnfNot0 cr))
       | OR -> BinL (AND, (nnfNot0 cl), (nnfNot0 cr)))
    | Not c0 -> nnf0 c0
    in nnf0
  
  (** val nnfNot :
      (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) -> t
      -> t **)
  
  let nnfNot altLt0 altNeq =
    let rec nnf0 c = match c with
    | Basic b -> c
    | Atom (op, tl, tr) ->
      (match op with
       | Lt -> altLt0 tl tr
       | Neq -> altNeq tl tr
       | _ -> Atom (op, tl, tr))
    | BinL (op, cl, cr) -> BinL (op, (nnf0 cl), (nnf0 cr))
    | Not c0 -> nnfNot0 c0
    and nnfNot0 = function
    | Basic b -> Basic (negb b)
    | Atom (op, tl, tr) ->
      (match op with
       | Eq -> altNeq tl tr
       | Le -> altLt0 tr tl
       | Lt -> Atom (Le, tr, tl)
       | Neq -> Atom (Eq, tl, tr))
    | BinL (op, cl, cr) ->
      (match op with
       | AND -> BinL (OR, (nnfNot0 cl), (nnfNot0 cr))
       | OR -> BinL (AND, (nnfNot0 cl), (nnfNot0 cr)))
    | Not c0 -> nnf0 c0
    in nnfNot0
  
  (** val altNeqByOR :
      (QTerm.term -> QTerm.term -> t) -> QTerm.term -> QTerm.term -> cond **)
  
  let altNeqByOR altLt0 t1 t2 =
    BinL (OR, (altLt0 t1 t2), (altLt0 t2 t1))
  
  (** val nnfForAssume : t -> t **)
  
  let nnfForAssume =
    nnf (fun x x0 -> Atom (Lt, x, x0))
      (let altLt0 = fun x x0 -> Atom (Lt, x, x0) in
       fun t1 t2 -> BinL (OR, (altLt0 t1 t2), (altLt0 t2 t1)))
  
  (** val nnfForRequire : t -> t **)
  
  let nnfForRequire =
    nnf (fun x x0 -> Atom (Lt, x, x0)) (fun x x0 -> Atom (Neq, x, x0))
  
  (** val lift : ZCond.t -> cond **)
  
  let rec lift = function
  | ZCond.Basic b -> Basic b
  | ZCond.Atom (oc, tl, tr) -> Atom (oc, (QTerm.lift tl), (QTerm.lift tr))
  | ZCond.BinL (op, cl, cr) -> BinL (op, (lift cl), (lift cr))
  | ZCond.Not c0 -> Not (lift c0)
  
  (** val toCstrX : QTerm.t -> CstrC.cmpT -> QTerm.t -> Cstr.t **)
  
  let toCstrX t1 cmp t2 =
    let lt1,aux1 = QTerm.toLinRec t1 QNum.u in
    let lt2,aux2 = QTerm.toLinRec t2 QNum.u in
    { Cstr.coefs = (Lin.add lt1 (Lin.neg lt2)); Cstr.typ = cmp; Cstr.cst =
    (QNum.sub aux2 aux1) }
  
  (** val toCstr : QTerm.t -> cmpT -> QTerm.t -> Cstr.t option **)
  
  let toCstr t1 cmp t2 =
    match cmp with
    | Eq ->
      Some
        (let lt1,aux1 = QTerm.toLinRec t1 QNum.u in
         let lt2,aux2 = QTerm.toLinRec t2 QNum.u in
         { Cstr.coefs = (Lin.add lt1 (Lin.neg lt2)); Cstr.typ = CstrC.Eq;
         Cstr.cst = (QNum.sub aux2 aux1) })
    | Le ->
      Some
        (let lt1,aux1 = QTerm.toLinRec t1 QNum.u in
         let lt2,aux2 = QTerm.toLinRec t2 QNum.u in
         { Cstr.coefs = (Lin.add lt1 (Lin.neg lt2)); Cstr.typ = CstrC.Le;
         Cstr.cst = (QNum.sub aux2 aux1) })
    | Lt ->
      Some
        (let lt1,aux1 = QTerm.toLinRec t1 QNum.u in
         let lt2,aux2 = QTerm.toLinRec t2 QNum.u in
         { Cstr.coefs = (Lin.add lt1 (Lin.neg lt2)); Cstr.typ = CstrC.Lt;
         Cstr.cst = (QNum.sub aux2 aux1) })
    | Neq -> None
 end

