open AffTerm
open BinNums
open CstrC
open Datatypes
open LinTerm
open NumC
open ProgVar

type cmpT =
| Eq
| Le
| Lt
| Neq

type binl =
| AND
| OR

module ZCond : 
 sig 
  module Term : 
   sig 
    type term = ZTerm.term =
    | Var of PVar.t
    | Cte of ZNum.t
    | Plus of term * term
    | Opp of term
    | Mult of term * ZNum.t
    | Old of term
    
    val term_rect :
      (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> ZNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1
    
    val term_rec :
      (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> ZNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1
    
    type t = term
    
    val eval : term -> ZNum.t Mem.t -> ZNum.t
    
    val xeval : term -> ZNum.t Mem.t -> ZNum.t Mem.t -> ZNum.t
    
    val mdBound : t -> PVar.t -> PVar.t
    
    val map : term -> PVar.t Mem.t -> term
    
    val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term
   end
  
  val cmpDenote_dec : cmpT -> ZNum.t -> ZNum.t -> bool
  
  type cond =
  | Basic of bool
  | Atom of cmpT * ZTerm.term * ZTerm.term
  | BinL of binl * cond * cond
  | Not of cond
  
  val cond_rect :
    (bool -> 'a1) -> (cmpT -> ZTerm.term -> ZTerm.term -> 'a1) -> (binl ->
    cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond -> 'a1
  
  val cond_rec :
    (bool -> 'a1) -> (cmpT -> ZTerm.term -> ZTerm.term -> 'a1) -> (binl ->
    cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond -> 'a1
  
  type t = cond
  
  val sat_dec : t -> ZNum.t Mem.t -> bool
  
  val xsat_dec : t -> ZNum.t Mem.t -> ZNum.t Mem.t -> bool
  
  val mdBound : cond -> positive -> positive
  
  val map : cond -> PVar.t Mem.t -> cond
  
  val xmap : cond -> PVar.t Mem.t -> PVar.t Mem.t -> cond
  
  val alt :
    (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) ->
    cmpT -> ZTerm.term -> ZTerm.term -> t
  
  val dual :
    (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) ->
    cmpT -> ZTerm.term -> ZTerm.term -> t
  
  val nnf :
    (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) -> t
    -> t
  
  val nnfNot :
    (ZTerm.term -> ZTerm.term -> t) -> (ZTerm.term -> ZTerm.term -> t) -> t
    -> t
  
  val altNeqByOR :
    (ZTerm.term -> ZTerm.term -> t) -> ZTerm.term -> ZTerm.term -> cond
  
  val altLt : ZTerm.term -> ZTerm.term -> cond
  
  val nnfForAssume : t -> t
  
  val nnfForRequire : t -> t
 end

module QCond : 
 sig 
  module Term : 
   sig 
    type term = QTerm.term =
    | Var of PVar.t
    | Cte of QNum.t
    | Plus of term * term
    | Opp of term
    | Mult of term * QNum.t
    | Old of term
    
    val term_rect :
      (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1
    
    val term_rec :
      (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1
    
    type t = term
    
    val eval : term -> QNum.t Mem.t -> QNum.t
    
    val xeval : term -> QNum.t Mem.t -> QNum.t Mem.t -> QNum.t
    
    val mdBound : t -> PVar.t -> PVar.t
    
    val map : term -> PVar.t Mem.t -> term
    
    val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term
    
    val lift : ZTerm.t -> term
    
    val toLinRec : t -> QNum.t -> Lin.t*QNum.t
    
    val toLin : t -> Lin.t*QNum.t
   end
  
  val cmpDenote_dec : cmpT -> QNum.t -> QNum.t -> bool
  
  type cond =
  | Basic of bool
  | Atom of cmpT * QTerm.term * QTerm.term
  | BinL of binl * cond * cond
  | Not of cond
  
  val cond_rect :
    (bool -> 'a1) -> (cmpT -> QTerm.term -> QTerm.term -> 'a1) -> (binl ->
    cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond -> 'a1
  
  val cond_rec :
    (bool -> 'a1) -> (cmpT -> QTerm.term -> QTerm.term -> 'a1) -> (binl ->
    cond -> 'a1 -> cond -> 'a1 -> 'a1) -> (cond -> 'a1 -> 'a1) -> cond -> 'a1
  
  type t = cond
  
  val sat_dec : t -> QNum.t Mem.t -> bool
  
  val xsat_dec : t -> QNum.t Mem.t -> QNum.t Mem.t -> bool
  
  val mdBound : cond -> positive -> positive
  
  val map : cond -> PVar.t Mem.t -> cond
  
  val xmap : cond -> PVar.t Mem.t -> PVar.t Mem.t -> cond
  
  val alt :
    (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) ->
    cmpT -> QTerm.term -> QTerm.term -> t
  
  val dual :
    (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) ->
    cmpT -> QTerm.term -> QTerm.term -> t
  
  val nnf :
    (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) -> t
    -> t
  
  val nnfNot :
    (QTerm.term -> QTerm.term -> t) -> (QTerm.term -> QTerm.term -> t) -> t
    -> t
  
  val altNeqByOR :
    (QTerm.term -> QTerm.term -> t) -> QTerm.term -> QTerm.term -> cond
  
  val nnfForAssume : t -> t
  
  val nnfForRequire : t -> t
  
  val lift : ZCond.t -> cond
  
  val toCstrX : QTerm.t -> CstrC.cmpT -> QTerm.t -> Cstr.t
  
  val toCstr : QTerm.t -> cmpT -> QTerm.t -> Cstr.t option
 end

