open LinTerm
open NumC
open ProgVar

module Term = 
 functor (N:NumSig) ->
 struct 
  type term =
  | Var of PVar.t
  | Cte of N.t
  | Plus of term * term
  | Opp of term
  | Mult of term * N.t
  | Old of term
  
  (** val term_rect :
      (PVar.t -> 'a1) -> (N.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
      -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> N.t -> 'a1) -> (term -> 'a1
      -> 'a1) -> term -> 'a1 **)
  
  let rec term_rect f f0 f1 f2 f3 f4 = function
  | Var x -> f x
  | Cte c -> f0 c
  | Plus (tl, tr) ->
    f1 tl (term_rect f f0 f1 f2 f3 f4 tl) tr (term_rect f f0 f1 f2 f3 f4 tr)
  | Opp te -> f2 te (term_rect f f0 f1 f2 f3 f4 te)
  | Mult (tl, c) -> f3 tl (term_rect f f0 f1 f2 f3 f4 tl) c
  | Old te -> f4 te (term_rect f f0 f1 f2 f3 f4 te)
  
  (** val term_rec :
      (PVar.t -> 'a1) -> (N.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
      -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> N.t -> 'a1) -> (term -> 'a1
      -> 'a1) -> term -> 'a1 **)
  
  let rec term_rec f f0 f1 f2 f3 f4 = function
  | Var x -> f x
  | Cte c -> f0 c
  | Plus (tl, tr) ->
    f1 tl (term_rec f f0 f1 f2 f3 f4 tl) tr (term_rec f f0 f1 f2 f3 f4 tr)
  | Opp te -> f2 te (term_rec f f0 f1 f2 f3 f4 te)
  | Mult (tl, c) -> f3 tl (term_rec f f0 f1 f2 f3 f4 tl) c
  | Old te -> f4 te (term_rec f f0 f1 f2 f3 f4 te)
  
  type t = term
  
  (** val eval : term -> N.t Mem.t -> N.t **)
  
  let rec eval te m =
    match te with
    | Var x -> m x
    | Cte c -> c
    | Plus (tl, tr) -> N.add (eval tl m) (eval tr m)
    | Opp te0 -> N.neg (eval te0 m)
    | Mult (tl, c) -> N.mul (eval tl m) c
    | Old te0 -> eval te0 m
  
  (** val xeval : term -> N.t Mem.t -> N.t Mem.t -> N.t **)
  
  let rec xeval te old new0 =
    match te with
    | Var x -> new0 x
    | Cte c -> c
    | Plus (tl, tr) -> N.add (xeval tl old new0) (xeval tr old new0)
    | Opp te0 -> N.neg (xeval te0 old new0)
    | Mult (tl, c) -> N.mul (xeval tl old new0) c
    | Old te0 -> eval te0 old
  
  (** val mdBound : t -> PVar.t -> PVar.t **)
  
  let rec mdBound te bound =
    match te with
    | Var x -> PVar.max bound x
    | Cte c -> bound
    | Plus (tl, tr) -> mdBound tl (mdBound tr bound)
    | Opp te0 -> mdBound te0 bound
    | Mult (tl, c) -> mdBound tl bound
    | Old te0 -> mdBound te0 bound
  
  (** val map : term -> PVar.t Mem.t -> term **)
  
  let rec map te f =
    match te with
    | Var x -> Var (f x)
    | Cte c -> te
    | Plus (tl, tr) -> Plus ((map tl f), (map tr f))
    | Opp te0 -> Opp (map te0 f)
    | Mult (te0, c) -> Mult ((map te0 f), c)
    | Old te0 -> map te0 f
  
  (** val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term **)
  
  let rec xmap te old new0 =
    match te with
    | Var x -> Var (new0 x)
    | Cte c -> te
    | Plus (tl, tr) -> Plus ((xmap tl old new0), (xmap tr old new0))
    | Opp te0 -> Opp (xmap te0 old new0)
    | Mult (te0, c) -> Mult ((xmap te0 old new0), c)
    | Old te0 -> Old (map te0 old)
 end

module ZTerm = Term(ZNum)

module QTerm = 
 struct 
  type term =
  | Var of PVar.t
  | Cte of QNum.t
  | Plus of term * term
  | Opp of term
  | Mult of term * QNum.t
  | Old of term
  
  (** val term_rect :
      (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1 **)
  
  let rec term_rect f f0 f1 f2 f3 f4 = function
  | Var x -> f x
  | Cte c -> f0 c
  | Plus (tl, tr) ->
    f1 tl (term_rect f f0 f1 f2 f3 f4 tl) tr (term_rect f f0 f1 f2 f3 f4 tr)
  | Opp te -> f2 te (term_rect f f0 f1 f2 f3 f4 te)
  | Mult (tl, c) -> f3 tl (term_rect f f0 f1 f2 f3 f4 tl) c
  | Old te -> f4 te (term_rect f f0 f1 f2 f3 f4 te)
  
  (** val term_rec :
      (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 ->
      'a1) -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term
      -> 'a1 -> 'a1) -> term -> 'a1 **)
  
  let rec term_rec f f0 f1 f2 f3 f4 = function
  | Var x -> f x
  | Cte c -> f0 c
  | Plus (tl, tr) ->
    f1 tl (term_rec f f0 f1 f2 f3 f4 tl) tr (term_rec f f0 f1 f2 f3 f4 tr)
  | Opp te -> f2 te (term_rec f f0 f1 f2 f3 f4 te)
  | Mult (tl, c) -> f3 tl (term_rec f f0 f1 f2 f3 f4 tl) c
  | Old te -> f4 te (term_rec f f0 f1 f2 f3 f4 te)
  
  type t = term
  
  (** val eval : term -> QNum.t Mem.t -> QNum.t **)
  
  let rec eval te m =
    match te with
    | Var x -> m x
    | Cte c -> c
    | Plus (tl, tr) -> QNum.add (eval tl m) (eval tr m)
    | Opp te0 -> QNum.neg (eval te0 m)
    | Mult (tl, c) -> QNum.mul (eval tl m) c
    | Old te0 -> eval te0 m
  
  (** val xeval : term -> QNum.t Mem.t -> QNum.t Mem.t -> QNum.t **)
  
  let rec xeval te old new0 =
    match te with
    | Var x -> new0 x
    | Cte c -> c
    | Plus (tl, tr) -> QNum.add (xeval tl old new0) (xeval tr old new0)
    | Opp te0 -> QNum.neg (xeval te0 old new0)
    | Mult (tl, c) -> QNum.mul (xeval tl old new0) c
    | Old te0 -> eval te0 old
  
  (** val mdBound : t -> PVar.t -> PVar.t **)
  
  let rec mdBound te bound =
    match te with
    | Var x -> PVar.max bound x
    | Cte c -> bound
    | Plus (tl, tr) -> mdBound tl (mdBound tr bound)
    | Opp te0 -> mdBound te0 bound
    | Mult (tl, c) -> mdBound tl bound
    | Old te0 -> mdBound te0 bound
  
  (** val map : term -> PVar.t Mem.t -> term **)
  
  let rec map te f =
    match te with
    | Var x -> Var (f x)
    | Cte c -> te
    | Plus (tl, tr) -> Plus ((map tl f), (map tr f))
    | Opp te0 -> Opp (map te0 f)
    | Mult (te0, c) -> Mult ((map te0 f), c)
    | Old te0 -> map te0 f
  
  (** val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term **)
  
  let rec xmap te old new0 =
    match te with
    | Var x -> Var (new0 x)
    | Cte c -> te
    | Plus (tl, tr) -> Plus ((xmap tl old new0), (xmap tr old new0))
    | Opp te0 -> Opp (xmap te0 old new0)
    | Mult (te0, c) -> Mult ((xmap te0 old new0), c)
    | Old te0 -> Old (map te0 old)
  
  (** val lift : ZTerm.t -> term **)
  
  let rec lift = function
  | ZTerm.Var x -> Var x
  | ZTerm.Cte c -> Cte (ZtoQ.ofZ c)
  | ZTerm.Plus (tl, tr) -> Plus ((lift tl), (lift tr))
  | ZTerm.Opp te0 -> Opp (lift te0)
  | ZTerm.Mult (tl, c) -> Mult ((lift tl), (ZtoQ.ofZ c))
  | ZTerm.Old te0 -> Old (lift te0)
  
  (** val toLinRec : t -> QNum.t -> Lin.t*QNum.t **)
  
  let rec toLinRec te c =
    match te with
    | Var x -> (Lin.single x c),QNum.z
    | Cte c0 -> Lin.nil,(QNum.mul c0 c)
    | Plus (tl, tr) ->
      let lt1,aux1 = toLinRec tl c in
      let lt2,aux2 = toLinRec tr c in (Lin.add lt1 lt2),(QNum.add aux1 aux2)
    | Opp t0 -> toLinRec t0 (QNum.neg c)
    | Mult (tl, c0) -> toLinRec tl (QNum.mul c0 c)
    | Old t0 -> toLinRec t0 c
  
  (** val toLin : t -> Lin.t*QNum.t **)
  
  let toLin te =
    toLinRec te QNum.u
 end

