open LinTerm
open NumC
open ProgVar

module Term : 
 functor (N:NumSig) ->
 sig 
  type term =
  | Var of PVar.t
  | Cte of N.t
  | Plus of term * term
  | Opp of term
  | Mult of term * N.t
  | Old of term
  
  val term_rect :
    (PVar.t -> 'a1) -> (N.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1) ->
    (term -> 'a1 -> 'a1) -> (term -> 'a1 -> N.t -> 'a1) -> (term -> 'a1 ->
    'a1) -> term -> 'a1
  
  val term_rec :
    (PVar.t -> 'a1) -> (N.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1) ->
    (term -> 'a1 -> 'a1) -> (term -> 'a1 -> N.t -> 'a1) -> (term -> 'a1 ->
    'a1) -> term -> 'a1
  
  type t = term
  
  val eval : term -> N.t Mem.t -> N.t
  
  val xeval : term -> N.t Mem.t -> N.t Mem.t -> N.t
  
  val mdBound : t -> PVar.t -> PVar.t
  
  val map : term -> PVar.t Mem.t -> term
  
  val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term
 end

module ZTerm : 
 sig 
  type term =
  | Var of PVar.t
  | Cte of ZNum.t
  | Plus of term * term
  | Opp of term
  | Mult of term * ZNum.t
  | Old of term
  
  val term_rect :
    (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
    -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> ZNum.t -> 'a1) -> (term -> 'a1
    -> 'a1) -> term -> 'a1
  
  val term_rec :
    (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
    -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> ZNum.t -> 'a1) -> (term -> 'a1
    -> 'a1) -> term -> 'a1
  
  type t = term
  
  val eval : term -> ZNum.t Mem.t -> ZNum.t
  
  val xeval : term -> ZNum.t Mem.t -> ZNum.t Mem.t -> ZNum.t
  
  val mdBound : t -> PVar.t -> PVar.t
  
  val map : term -> PVar.t Mem.t -> term
  
  val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term
 end

module QTerm : 
 sig 
  type term =
  | Var of PVar.t
  | Cte of QNum.t
  | Plus of term * term
  | Opp of term
  | Mult of term * QNum.t
  | Old of term
  
  val term_rect :
    (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
    -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term -> 'a1
    -> 'a1) -> term -> 'a1
  
  val term_rec :
    (PVar.t -> 'a1) -> (QNum.t -> 'a1) -> (term -> 'a1 -> term -> 'a1 -> 'a1)
    -> (term -> 'a1 -> 'a1) -> (term -> 'a1 -> QNum.t -> 'a1) -> (term -> 'a1
    -> 'a1) -> term -> 'a1
  
  type t = term
  
  val eval : term -> QNum.t Mem.t -> QNum.t
  
  val xeval : term -> QNum.t Mem.t -> QNum.t Mem.t -> QNum.t
  
  val mdBound : t -> PVar.t -> PVar.t
  
  val map : term -> PVar.t Mem.t -> term
  
  val xmap : term -> PVar.t Mem.t -> PVar.t Mem.t -> term
  
  val lift : ZTerm.t -> term
  
  val toLinRec : t -> QNum.t -> Lin.t*QNum.t
  
  val toLin : t -> Lin.t*QNum.t
 end

