open NumC

module Cert = 
 struct 
  type frag_t = (int*QNum.t) list
  
  type cstrT =
  | Direct of frag_t
  | SplitEq of frag_t * frag_t
  
  (** val cstrT_rect :
      (frag_t -> 'a1) -> (frag_t -> frag_t -> 'a1) -> cstrT -> 'a1 **)
  
  let cstrT_rect f f0 = function
  | Direct x -> f x
  | SplitEq (x, x0) -> f0 x x0
  
  (** val cstrT_rec :
      (frag_t -> 'a1) -> (frag_t -> frag_t -> 'a1) -> cstrT -> 'a1 **)
  
  let cstrT_rec f f0 = function
  | Direct x -> f x
  | SplitEq (x, x0) -> f0 x x0
  
  type finalT =
  | Implies of (int*cstrT) list
  | Empty of frag_t
  
  (** val finalT_rect :
      ((int*cstrT) list -> 'a1) -> (frag_t -> 'a1) -> finalT -> 'a1 **)
  
  let finalT_rect f f0 = function
  | Implies x -> f x
  | Empty x -> f0 x
  
  (** val finalT_rec :
      ((int*cstrT) list -> 'a1) -> (frag_t -> 'a1) -> finalT -> 'a1 **)
  
  let finalT_rec f f0 = function
  | Implies x -> f x
  | Empty x -> f0 x
  
  type t =
  | Bind of int * cstrT * t
  | Final of finalT
  
  (** val t_rect :
      (int -> cstrT -> t -> 'a1 -> 'a1) -> (finalT -> 'a1) -> t -> 'a1 **)
  
  let rec t_rect f f0 = function
  | Bind (i, c, t1) -> f i c t1 (t_rect f f0 t1)
  | Final f1 -> f0 f1
  
  (** val t_rec :
      (int -> cstrT -> t -> 'a1 -> 'a1) -> (finalT -> 'a1) -> t -> 'a1 **)
  
  let rec t_rec f f0 = function
  | Bind (i, c, t1) -> f i c t1 (t_rec f f0 t1)
  | Final f1 -> f0 f1
  
  type joinEltT = { id : int; arg1 : cstrT; arg2 : cstrT }
  
  (** val joinEltT_rect :
      (int -> cstrT -> cstrT -> 'a1) -> joinEltT -> 'a1 **)
  
  let joinEltT_rect f j =
    let { id = x; arg1 = x0; arg2 = x1 } = j in f x x0 x1
  
  (** val joinEltT_rec :
      (int -> cstrT -> cstrT -> 'a1) -> joinEltT -> 'a1 **)
  
  let joinEltT_rec f j =
    let { id = x; arg1 = x0; arg2 = x1 } = j in f x x0 x1
  
  (** val id : joinEltT -> int **)
  
  let id x = x.id
  
  (** val arg1 : joinEltT -> cstrT **)
  
  let arg1 x = x.arg1
  
  (** val arg2 : joinEltT -> cstrT **)
  
  let arg2 x = x.arg2
  
  type joinT = joinEltT list
  
  type asgT =
  | AsgInv of (int*t) list * t
  | AsgNInv of t * (int*t) list
  
  (** val asgT_rect :
      ((int*t) list -> t -> 'a1) -> (t -> (int*t) list -> 'a1) -> asgT -> 'a1 **)
  
  let asgT_rect f f0 = function
  | AsgInv (x, x0) -> f x x0
  | AsgNInv (x, x0) -> f0 x x0
  
  (** val asgT_rec :
      ((int*t) list -> t -> 'a1) -> (t -> (int*t) list -> 'a1) -> asgT -> 'a1 **)
  
  let asgT_rec f f0 = function
  | AsgInv (x, x0) -> f x x0
  | AsgNInv (x, x0) -> f0 x x0
  
  type meetT = { sh : int; meetCert : t }
  
  (** val meetT_rect : (int -> t -> 'a1) -> meetT -> 'a1 **)
  
  let meetT_rect f m =
    let { sh = x; meetCert = x0 } = m in f x x0
  
  (** val meetT_rec : (int -> t -> 'a1) -> meetT -> 'a1 **)
  
  let meetT_rec f m =
    let { sh = x; meetCert = x0 } = m in f x x0
  
  (** val sh : meetT -> int **)
  
  let sh x = x.sh
  
  (** val meetCert : meetT -> t **)
  
  let meetCert x = x.meetCert
 end

