open LinTerm
open NumC
open ProgVar
open String0

type cmpT =
| Eq
| Le
| Lt

module Cstr = 
 struct 
  type tInd = { coefs : Lin.t; typ : cmpT; cst : QNum.t }
  
  (** val tInd_rect : (Lin.t -> cmpT -> QNum.t -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rect f t0 =
    let { coefs = x; typ = x0; cst = x1 } = t0 in f x x0 x1
  
  (** val tInd_rec : (Lin.t -> cmpT -> QNum.t -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rec f t0 =
    let { coefs = x; typ = x0; cst = x1 } = t0 in f x x0 x1
  
  (** val coefs : tInd -> Lin.t **)
  
  let coefs x = x.coefs
  
  (** val typ : tInd -> cmpT **)
  
  let typ x = x.typ
  
  (** val cst : tInd -> QNum.t **)
  
  let cst x = x.cst
  
  type t = tInd
  
  (** val top : tInd **)
  
  let top =
    { coefs = Lin.nil; typ = Eq; cst = QNum.z }
  
  (** val isInclEq : t -> t -> bool **)
  
  let isInclEq c1 c2 =
    match c2.typ with
    | Eq -> if QNum.eqDec c1.cst c2.cst then true else false
    | Le -> if QNum.ltLeDec c2.cst c1.cst then false else true
    | Lt -> if QNum.ltLeDec c1.cst c2.cst then true else false
  
  (** val isInclLt : t -> t -> bool **)
  
  let isInclLt c1 c2 =
    match c2.typ with
    | Eq -> false
    | _ -> if QNum.ltLeDec c2.cst c1.cst then false else true
  
  (** val isInclLe : t -> t -> bool **)
  
  let isInclLe c1 c2 =
    match c2.typ with
    | Eq -> false
    | Le -> if QNum.ltLeDec c2.cst c1.cst then false else true
    | Lt -> if QNum.ltLeDec c1.cst c2.cst then true else false
  
  (** val isIncl : t -> t -> bool **)
  
  let isIncl c1 c2 =
    if Lin.isEq c1.coefs c2.coefs
    then (match c1.typ with
          | Eq ->
            (match c2.typ with
             | Eq -> if QNum.eqDec c1.cst c2.cst then true else false
             | Le -> if QNum.ltLeDec c2.cst c1.cst then false else true
             | Lt -> if QNum.ltLeDec c1.cst c2.cst then true else false)
          | Le ->
            (match c2.typ with
             | Eq -> false
             | Le -> if QNum.ltLeDec c2.cst c1.cst then false else true
             | Lt -> if QNum.ltLeDec c1.cst c2.cst then true else false)
          | Lt ->
            (match c2.typ with
             | Eq -> false
             | _ -> if QNum.ltLeDec c2.cst c1.cst then false else true))
    else false
  
  (** val isContrad : t -> bool **)
  
  let isContrad c =
    if Lin.isEq c.coefs Lin.nil
    then (match c.typ with
          | Eq -> if QNum.eqDec QNum.z c.cst then false else true
          | Le -> if QNum.ltLeDec c.cst QNum.z then true else false
          | Lt -> if QNum.ltLeDec QNum.z c.cst then false else true)
    else false
  
  (** val cmpAdd : cmpT -> cmpT -> cmpT **)
  
  let cmpAdd t1 t2 =
    match t1 with
    | Eq -> t2
    | Le ->
      (match t2 with
       | Eq -> Le
       | x -> x)
    | Lt -> Lt
  
  (** val add : t -> t -> t **)
  
  let add c1 c2 =
    { coefs = (Lin.add c1.coefs c2.coefs); typ = (cmpAdd c1.typ c2.typ);
      cst = (QNum.add c1.cst c2.cst) }
  
  (** val mulSimpl : t -> QNum.t -> t **)
  
  let mulSimpl c n =
    { coefs = (Lin.mul n c.coefs); typ = c.typ; cst = (QNum.mul n c.cst) }
  
  (** val mul : QNum.t -> t -> t **)
  
  let mul n c =
    match c.typ with
    | Eq -> mulSimpl c n
    | _ ->
      if QNum.ltLeDec QNum.z n
      then mulSimpl c n
      else (fun l _ -> failwith (CoqPr.charListTr l)) ('m'::('u'::('l'::[])))
             top
  
  (** val merge : tInd -> tInd -> t **)
  
  let merge c1 c2 =
    match c1.typ with
    | Le ->
      (match c2.typ with
       | Le ->
         if Lin.isEq c2.coefs (Lin.neg c1.coefs)
         then if QNum.eqDec c2.cst (QNum.neg c1.cst)
              then { coefs = c1.coefs; typ = Eq; cst = c1.cst }
              else (fun l _ -> failwith (CoqPr.charListTr l))
                     ('m'::('e'::('r'::('g'::('e'::('_'::('1'::[]))))))) top
         else (fun l _ -> failwith (CoqPr.charListTr l))
                ('m'::('e'::('r'::('g'::('e'::('_'::('2'::[]))))))) top
       | _ ->
         (fun l _ -> failwith (CoqPr.charListTr l))
           ('m'::('e'::('r'::('g'::('e'::('_'::('3'::[]))))))) top)
    | _ ->
      (fun l _ -> failwith (CoqPr.charListTr l))
        ('m'::('e'::('r'::('g'::('e'::('_'::('4'::[]))))))) top
  
  (** val split : t -> (t*t) option **)
  
  let split c =
    let { coefs = a; typ = typ0; cst = b } = c in
    (match typ0 with
     | Eq ->
       Some ({ coefs = a; typ = Le; cst = b },{ coefs = (Lin.neg a); typ =
         Le; cst = (QNum.neg b) })
     | _ -> None)
  
  (** val isFree : PVar.t -> t -> bool **)
  
  let isFree x c =
    Lin.isFree x c.coefs
  
  (** val rename : PVar.t -> PVar.t -> tInd -> t **)
  
  let rename x y c =
    { coefs = (Lin.rename x y c.coefs); typ = c.typ; cst = c.cst }
  
  (** val cmpPr : cmpT -> char list **)
  
  let cmpPr = function
  | Eq -> '='::[]
  | Le -> '<'::('='::[])
  | Lt -> '<'::[]
  
  (** val pr : t -> char list **)
  
  let pr c =
    append (Lin.pr c.coefs)
      (append (' '::[])
        (append (cmpPr c.typ) (append (' '::[]) (QNum.pr c.cst))))
  
  (** val upperToCstr : Lin.t -> QNum.t -> t **)
  
  let upperToCstr v n =
    { coefs = v; typ = Lt; cst = n }
  
  (** val upperOrEqualsToCstr : Lin.t -> QNum.t -> t **)
  
  let upperOrEqualsToCstr v n =
    { coefs = v; typ = Le; cst = n }
  
  (** val lowerToCstr : Lin.t -> QNum.t -> t **)
  
  let lowerToCstr v n =
    { coefs = (Lin.neg v); typ = Lt; cst = (QNum.neg n) }
  
  (** val lowerOrEqualsToCstr : Lin.t -> QNum.t -> t **)
  
  let lowerOrEqualsToCstr v n =
    { coefs = (Lin.neg v); typ = Le; cst = (QNum.neg n) }
 end

