open BinNums
open Datatypes
open OrderedTypeEx
open Peano

(** val append : positive -> positive -> positive **)

let rec append i j =
  match i with
  | Coq_xI ii -> Coq_xI (append ii j)
  | Coq_xO ii -> Coq_xO (append ii j)
  | Coq_xH -> j

module PositiveMap = 
 struct 
  module E = PositiveOrderedTypeBits
  
  module ME = OrderedType.KeyOrderedType(E)
  
  type key = positive
  
  type 'a tree =
  | Leaf
  | Node of 'a tree * 'a option * 'a tree
  
  type 'a t = 'a tree
  
  (** val empty : 'a1 t **)
  
  let empty =
    Leaf
  
  (** val is_empty : 'a1 t -> bool **)
  
  let rec is_empty = function
  | Leaf -> true
  | Node (l, o, r) ->
    (match o with
     | Some a -> false
     | None -> if is_empty l then is_empty r else false)
  
  (** val find : positive -> 'a1 t -> 'a1 option **)
  
  let rec find i = function
  | Leaf -> None
  | Node (l, o, r) ->
    (match i with
     | Coq_xI ii -> find ii r
     | Coq_xO ii -> find ii l
     | Coq_xH -> o)
  
  (** val mem : positive -> 'a1 t -> bool **)
  
  let rec mem i = function
  | Leaf -> false
  | Node (l, o, r) ->
    (match i with
     | Coq_xI ii -> mem ii r
     | Coq_xO ii -> mem ii l
     | Coq_xH ->
       (match o with
        | Some a -> true
        | None -> false))
  
  (** val add : positive -> 'a1 -> 'a1 t -> 'a1 t **)
  
  let rec add i v = function
  | Leaf ->
    (match i with
     | Coq_xI ii -> Node (Leaf, None, (add ii v Leaf))
     | Coq_xO ii -> Node ((add ii v Leaf), None, Leaf)
     | Coq_xH -> Node (Leaf, (Some v), Leaf))
  | Node (l, o, r) ->
    (match i with
     | Coq_xI ii -> Node (l, o, (add ii v r))
     | Coq_xO ii -> Node ((add ii v l), o, r)
     | Coq_xH -> Node (l, (Some v), r))
  
  (** val remove : positive -> 'a1 t -> 'a1 t **)
  
  let rec remove i m =
    match i with
    | Coq_xI ii ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match l with
          | Leaf ->
            (match o with
             | Some a -> Node (l, o, (remove ii r))
             | None ->
               (match remove ii r with
                | Leaf -> Leaf
                | Node (t0, o0, t1) -> Node (Leaf, None, (Node (t0, o0, t1)))))
          | Node (t0, o0, t1) -> Node (l, o, (remove ii r))))
    | Coq_xO ii ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match o with
          | Some a -> Node ((remove ii l), o, r)
          | None ->
            (match r with
             | Leaf ->
               (match remove ii l with
                | Leaf -> Leaf
                | Node (t0, o0, t1) -> Node ((Node (t0, o0, t1)), None, Leaf))
             | Node (t0, o0, t1) -> Node ((remove ii l), o, r))))
    | Coq_xH ->
      (match m with
       | Leaf -> Leaf
       | Node (l, o, r) ->
         (match l with
          | Leaf ->
            (match r with
             | Leaf -> Leaf
             | Node (t0, o0, t1) -> Node (l, None, r))
          | Node (t0, o0, t1) -> Node (l, None, r)))
  
  (** val xelements : 'a1 t -> positive -> (positive*'a1) list **)
  
  let rec xelements m i =
    match m with
    | Leaf -> []
    | Node (l, o, r) ->
      (match o with
       | Some x ->
         app (xelements l (append i (Coq_xO Coq_xH)))
           ((i,x)::(xelements r (append i (Coq_xI Coq_xH))))
       | None ->
         app (xelements l (append i (Coq_xO Coq_xH)))
           (xelements r (append i (Coq_xI Coq_xH))))
  
  (** val elements : 'a1 t -> (positive*'a1) list **)
  
  let elements m =
    xelements m Coq_xH
  
  (** val cardinal : 'a1 t -> nat **)
  
  let rec cardinal = function
  | Leaf -> O
  | Node (l, o, r) ->
    (match o with
     | Some a -> S (plus (cardinal l) (cardinal r))
     | None -> plus (cardinal l) (cardinal r))
  
  (** val xfind : positive -> positive -> 'a1 t -> 'a1 option **)
  
  let rec xfind i j m =
    match i with
    | Coq_xI ii ->
      (match j with
       | Coq_xI jj -> xfind ii jj m
       | Coq_xO p -> None
       | Coq_xH -> find i m)
    | Coq_xO ii ->
      (match j with
       | Coq_xI p -> None
       | Coq_xO jj -> xfind ii jj m
       | Coq_xH -> find i m)
    | Coq_xH ->
      (match j with
       | Coq_xH -> find i m
       | _ -> None)
  
  (** val xmapi : (positive -> 'a1 -> 'a2) -> 'a1 t -> positive -> 'a2 t **)
  
  let rec xmapi f m i =
    match m with
    | Leaf -> Leaf
    | Node (l, o, r) ->
      Node ((xmapi f l (append i (Coq_xO Coq_xH))), (option_map (f i) o),
        (xmapi f r (append i (Coq_xI Coq_xH))))
  
  (** val mapi : (positive -> 'a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let mapi f m =
    xmapi f m Coq_xH
  
  (** val map : ('a1 -> 'a2) -> 'a1 t -> 'a2 t **)
  
  let map f m =
    mapi (fun x -> f) m
  
  (** val xmap2_l :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a1 t -> 'a3 t **)
  
  let rec xmap2_l f = function
  | Leaf -> Leaf
  | Node (l, o, r) -> Node ((xmap2_l f l), (f o None), (xmap2_l f r))
  
  (** val xmap2_r :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a2 t -> 'a3 t **)
  
  let rec xmap2_r f = function
  | Leaf -> Leaf
  | Node (l, o, r) -> Node ((xmap2_r f l), (f None o), (xmap2_r f r))
  
  (** val _map2 :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a1 t -> 'a2 t -> 'a3 t **)
  
  let rec _map2 f m1 m2 =
    match m1 with
    | Leaf -> xmap2_r f m2
    | Node (l1, o1, r1) ->
      (match m2 with
       | Leaf -> xmap2_l f m1
       | Node (l2, o2, r2) ->
         Node ((_map2 f l1 l2), (f o1 o2), (_map2 f r1 r2)))
  
  (** val map2 :
      ('a1 option -> 'a2 option -> 'a3 option) -> 'a1 t -> 'a2 t -> 'a3 t **)
  
  let map2 f =
    _map2 (fun o1 o2 ->
      match o1 with
      | Some y -> f o1 o2
      | None ->
        (match o2 with
         | Some y -> f o1 o2
         | None -> None))
  
  (** val xfoldi :
      (positive -> 'a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> positive -> 'a2 **)
  
  let rec xfoldi f m v i =
    match m with
    | Leaf -> v
    | Node (l, o, r) ->
      (match o with
       | Some x ->
         xfoldi f r (f i x (xfoldi f l v (append i (Coq_xO Coq_xH))))
           (append i (Coq_xI Coq_xH))
       | None ->
         xfoldi f r (xfoldi f l v (append i (Coq_xO Coq_xH)))
           (append i (Coq_xI Coq_xH)))
  
  (** val fold : (positive -> 'a1 -> 'a2 -> 'a2) -> 'a1 t -> 'a2 -> 'a2 **)
  
  let fold f m i =
    xfoldi f m i Coq_xH
  
  (** val equal : ('a1 -> 'a1 -> bool) -> 'a1 t -> 'a1 t -> bool **)
  
  let rec equal cmp m1 m2 =
    match m1 with
    | Leaf -> is_empty m2
    | Node (l1, o1, r1) ->
      (match m2 with
       | Leaf -> is_empty m1
       | Node (l2, o2, r2) ->
         if if match o1 with
               | Some v1 ->
                 (match o2 with
                  | Some v2 -> cmp v1 v2
                  | None -> false)
               | None ->
                 (match o2 with
                  | Some a -> false
                  | None -> true)
            then equal cmp l1 l2
            else false
         then equal cmp r1 r2
         else false)
 end

