open Datatypes
open NumC
open Specif
open String0

type __ = Obj.t
let __ = let rec f _ = Obj.repr f in Obj.repr f

module type ItvSig = 
 functor (N:NumSig) ->
 sig 
  type t 
  
  val mk1Dep : N.t -> t
  
  val mk1 : N.t -> t
  
  val addDep : t -> t -> t
  
  val add : t -> t -> t
  
  val negDep : t -> t
  
  val neg : t -> t
  
  val mulcDep : t -> N.t -> t
  
  val mulc : t -> N.t -> t
  
  val pr : t -> char list
 end

module StdItv = 
 functor (N:NumSig) ->
 struct 
  type bndT =
  | Infty
  | Open of N.t
  | Closed of N.t
  
  (** val bndT_rect : 'a1 -> (N.t -> 'a1) -> (N.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rect f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  (** val bndT_rec : 'a1 -> (N.t -> 'a1) -> (N.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rec f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  type tInd =
  | Bot
  | NotBot of bndT * bndT
  
  (** val tInd_rect : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rect f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  (** val tInd_rec : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rec f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  type t = tInd
  
  (** val mk : bndT -> bndT -> t **)
  
  let mk l u =
    match l with
    | Infty -> NotBot (Infty, u)
    | Open n1 ->
      (match u with
       | Infty -> NotBot ((Open n1), Infty)
       | Open n2 ->
         if N.ltLeDec n1 n2 then NotBot ((Open n1), (Open n2)) else Bot
       | Closed n2 ->
         if N.ltLeDec n1 n2 then NotBot ((Open n1), (Closed n2)) else Bot)
    | Closed n1 ->
      (match u with
       | Infty -> NotBot ((Closed n1), Infty)
       | Open n2 ->
         if N.ltLeDec n1 n2 then NotBot ((Closed n1), (Open n2)) else Bot
       | Closed n2 ->
         if N.ltLeDec n2 n1 then Bot else NotBot ((Closed n1), (Closed n2)))
  
  (** val mk1Dep : N.t -> t **)
  
  let mk1Dep n =
    NotBot ((Closed n), (Closed n))
  
  (** val mk1 : N.t -> t **)
  
  let mk1 n =
    mk1Dep n
  
  (** val bndAdd : bndT -> bndT -> bndT **)
  
  let bndAdd b1 b2 =
    match b1 with
    | Infty -> Infty
    | Open n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (N.add n1 n2)
       | Closed n2 -> Open (N.add n1 n2))
    | Closed n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (N.add n1 n2)
       | Closed n2 -> Closed (N.add n1 n2))
  
  (** val addDep : t -> t -> t **)
  
  let addDep i1 i2 =
    match i1 with
    | Bot -> Bot
    | NotBot (l1, u1) ->
      (match i2 with
       | Bot -> Bot
       | NotBot (l2, u2) -> NotBot ((bndAdd l1 l2), (bndAdd u1 u2)))
  
  (** val add : t -> t -> t **)
  
  let add i1 i2 =
    addDep i1 i2
  
  (** val bndNeg : bndT -> bndT **)
  
  let bndNeg = function
  | Infty -> Infty
  | Open n -> Open (N.neg n)
  | Closed n -> Closed (N.neg n)
  
  (** val negDep : t -> t **)
  
  let negDep = function
  | Bot -> Bot
  | NotBot (l, u) -> NotBot ((bndNeg u), (bndNeg l))
  
  (** val neg : t -> t **)
  
  let neg i =
    negDep i
  
  (** val bndMulc : bndT -> N.t -> bndT **)
  
  let bndMulc i n =
    match i with
    | Infty -> Infty
    | Open n' -> Open (N.mul n n')
    | Closed n' -> Closed (N.mul n n')
  
  (** val mulcPos : t -> N.t -> t **)
  
  let mulcPos i0 n0 =
    match i0 with
    | Bot -> Bot
    | NotBot (l, u) -> NotBot ((bndMulc l n0), (bndMulc u n0))
  
  (** val mulcZ : t -> t **)
  
  let mulcZ = function
  | Bot -> Bot
  | NotBot (l, u) ->
    (match l with
     | Infty ->
       (match u with
        | Infty -> NotBot (Infty, Infty)
        | _ -> NotBot (Infty, (Closed N.z)))
     | _ ->
       (match u with
        | Infty -> NotBot ((Closed N.z), Infty)
        | _ -> NotBot ((Closed N.z), (Closed N.z))))
  
  (** val mulcDep : t -> N.t -> t **)
  
  let mulcDep i0 n =
    match N.dec N.z n with
    | Coq_inleft pfn0 ->
      if pfn0 then mulcPos i0 n else negDep (mulcPos i0 (N.neg n))
    | Coq_inright -> mulcZ i0
  
  (** val mulc : t -> N.t -> t **)
  
  let mulc i n =
    mulcDep i n
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Bot -> 'b'::('o'::('t'::[]))
  | NotBot (l, u) ->
    let lstr =
      match l with
      | Infty -> ']'::('-'::('i'::('n'::('f'::('t'::('y'::[]))))))
      | Open n -> append (']'::[]) (N.pr n)
      | Closed n -> append ('['::[]) (N.pr n)
    in
    let ustr =
      match u with
      | Infty -> '+'::('i'::('n'::('f'::('t'::('y'::('['::[]))))))
      | Open n -> append (N.pr n) ('['::[])
      | Closed n -> append (N.pr n) (']'::[])
    in
    append lstr (append (','::(' '::[])) ustr)
 end

module QItv = 
 struct 
  type bndT =
  | Infty
  | Open of QNum.t
  | Closed of QNum.t
  
  (** val bndT_rect :
      'a1 -> (QNum.t -> 'a1) -> (QNum.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rect f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  (** val bndT_rec :
      'a1 -> (QNum.t -> 'a1) -> (QNum.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rec f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  type tInd =
  | Bot
  | NotBot of bndT * bndT
  
  (** val tInd_rect : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rect f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  (** val tInd_rec : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rec f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  type t = tInd
  
  (** val mk : bndT -> bndT -> t **)
  
  let mk l u =
    match l with
    | Infty -> NotBot (Infty, u)
    | Open n1 ->
      (match u with
       | Infty -> NotBot ((Open n1), Infty)
       | Open n2 ->
         if QNum.ltLeDec n1 n2 then NotBot ((Open n1), (Open n2)) else Bot
       | Closed n2 ->
         if QNum.ltLeDec n1 n2 then NotBot ((Open n1), (Closed n2)) else Bot)
    | Closed n1 ->
      (match u with
       | Infty -> NotBot ((Closed n1), Infty)
       | Open n2 ->
         if QNum.ltLeDec n1 n2 then NotBot ((Closed n1), (Open n2)) else Bot
       | Closed n2 ->
         if QNum.ltLeDec n2 n1
         then Bot
         else NotBot ((Closed n1), (Closed n2)))
  
  (** val mk1Dep : QNum.t -> t **)
  
  let mk1Dep n =
    NotBot ((Closed n), (Closed n))
  
  (** val mk1 : QNum.t -> t **)
  
  let mk1 n =
    mk1Dep n
  
  (** val bndAdd : bndT -> bndT -> bndT **)
  
  let bndAdd b1 b2 =
    match b1 with
    | Infty -> Infty
    | Open n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (QNum.add n1 n2)
       | Closed n2 -> Open (QNum.add n1 n2))
    | Closed n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (QNum.add n1 n2)
       | Closed n2 -> Closed (QNum.add n1 n2))
  
  (** val addDep : t -> t -> t **)
  
  let addDep i1 i2 =
    match i1 with
    | Bot -> Bot
    | NotBot (l1, u1) ->
      (match i2 with
       | Bot -> Bot
       | NotBot (l2, u2) -> NotBot ((bndAdd l1 l2), (bndAdd u1 u2)))
  
  (** val add : t -> t -> t **)
  
  let add i1 i2 =
    addDep i1 i2
  
  (** val bndNeg : bndT -> bndT **)
  
  let bndNeg = function
  | Infty -> Infty
  | Open n -> Open (QNum.neg n)
  | Closed n -> Closed (QNum.neg n)
  
  (** val negDep : t -> t **)
  
  let negDep = function
  | Bot -> Bot
  | NotBot (l, u) -> NotBot ((bndNeg u), (bndNeg l))
  
  (** val neg : t -> t **)
  
  let neg i =
    negDep i
  
  (** val bndMulc : bndT -> QNum.t -> bndT **)
  
  let bndMulc i n =
    match i with
    | Infty -> Infty
    | Open n' -> Open (QNum.mul n n')
    | Closed n' -> Closed (QNum.mul n n')
  
  (** val mulcPos : t -> QNum.t -> t **)
  
  let mulcPos i0 n0 =
    match i0 with
    | Bot -> Bot
    | NotBot (l, u) -> NotBot ((bndMulc l n0), (bndMulc u n0))
  
  (** val mulcZ : t -> t **)
  
  let mulcZ = function
  | Bot -> Bot
  | NotBot (l, u) ->
    (match l with
     | Infty ->
       (match u with
        | Infty -> NotBot (Infty, Infty)
        | _ -> NotBot (Infty, (Closed QNum.z)))
     | _ ->
       (match u with
        | Infty -> NotBot ((Closed QNum.z), Infty)
        | _ -> NotBot ((Closed QNum.z), (Closed QNum.z))))
  
  (** val mulcDep : t -> QNum.t -> t **)
  
  let mulcDep i0 n =
    match QNum.dec QNum.z n with
    | Coq_inleft pfn0 ->
      if pfn0 then mulcPos i0 n else negDep (mulcPos i0 (QNum.neg n))
    | Coq_inright -> mulcZ i0
  
  (** val mulc : t -> QNum.t -> t **)
  
  let mulc i n =
    mulcDep i n
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Bot -> 'b'::('o'::('t'::[]))
  | NotBot (l, u) ->
    let lstr =
      match l with
      | Infty -> ']'::('-'::('i'::('n'::('f'::('t'::('y'::[]))))))
      | Open n -> append (']'::[]) (QNum.pr n)
      | Closed n -> append ('['::[]) (QNum.pr n)
    in
    let ustr =
      match u with
      | Infty -> '+'::('i'::('n'::('f'::('t'::('y'::('['::[]))))))
      | Open n -> append (QNum.pr n) ('['::[])
      | Closed n -> append (QNum.pr n) (']'::[])
    in
    append lstr (append (','::(' '::[])) ustr)
  
  (** val shiftDep : t -> QNum.t -> t **)
  
  let shiftDep i0 n =
    match i0 with
    | Bot -> Bot
    | NotBot (low, up) ->
      let i1 =
        match low with
        | Infty ->
          (match up with
           | Infty -> Infty,Infty
           | Open n1 -> Infty,(Open (QNum.add n n1))
           | Closed n1 -> Infty,(Closed (QNum.add n n1)))
        | Open n1 ->
          (match up with
           | Infty -> (Open (QNum.add n n1)),Infty
           | Open n2 -> (Open (QNum.add n n1)),(Open (QNum.add n n2))
           | Closed n2 -> (Open (QNum.add n n1)),(Closed (QNum.add n n2)))
        | Closed n1 ->
          (match up with
           | Infty -> (Closed (QNum.add n n1)),Infty
           | Open n2 -> (Closed (QNum.add n n1)),(Open (QNum.add n n2))
           | Closed n2 -> (Closed (QNum.add n n1)),(Closed (QNum.add n n2)))
      in
      NotBot ((fst i1), (snd i1))
  
  (** val shift : t -> QNum.t -> t **)
  
  let shift i n =
    shiftDep i n
 end

module ZItv = 
 struct 
  type bndT =
  | Infty
  | Open of ZNum.t
  | Closed of ZNum.t
  
  (** val bndT_rect :
      'a1 -> (ZNum.t -> 'a1) -> (ZNum.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rect f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  (** val bndT_rec :
      'a1 -> (ZNum.t -> 'a1) -> (ZNum.t -> 'a1) -> bndT -> 'a1 **)
  
  let bndT_rec f f0 f1 = function
  | Infty -> f
  | Open x -> f0 x
  | Closed x -> f1 x
  
  type tInd =
  | Bot
  | NotBot of bndT * bndT
  
  (** val tInd_rect : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rect f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  (** val tInd_rec : 'a1 -> (bndT -> bndT -> __ -> 'a1) -> tInd -> 'a1 **)
  
  let tInd_rec f f0 = function
  | Bot -> f
  | NotBot (x, x0) -> f0 x x0 __
  
  type t = tInd
  
  (** val mk : bndT -> bndT -> t **)
  
  let mk l u =
    match l with
    | Infty -> NotBot (Infty, u)
    | Open n1 ->
      (match u with
       | Infty -> NotBot ((Open n1), Infty)
       | Open n2 ->
         if ZNum.ltLeDec n1 n2 then NotBot ((Open n1), (Open n2)) else Bot
       | Closed n2 ->
         if ZNum.ltLeDec n1 n2 then NotBot ((Open n1), (Closed n2)) else Bot)
    | Closed n1 ->
      (match u with
       | Infty -> NotBot ((Closed n1), Infty)
       | Open n2 ->
         if ZNum.ltLeDec n1 n2 then NotBot ((Closed n1), (Open n2)) else Bot
       | Closed n2 ->
         if ZNum.ltLeDec n2 n1
         then Bot
         else NotBot ((Closed n1), (Closed n2)))
  
  (** val mk1Dep : ZNum.t -> t **)
  
  let mk1Dep n =
    NotBot ((Closed n), (Closed n))
  
  (** val mk1 : ZNum.t -> t **)
  
  let mk1 n =
    mk1Dep n
  
  (** val bndAdd : bndT -> bndT -> bndT **)
  
  let bndAdd b1 b2 =
    match b1 with
    | Infty -> Infty
    | Open n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (ZNum.add n1 n2)
       | Closed n2 -> Open (ZNum.add n1 n2))
    | Closed n1 ->
      (match b2 with
       | Infty -> Infty
       | Open n2 -> Open (ZNum.add n1 n2)
       | Closed n2 -> Closed (ZNum.add n1 n2))
  
  (** val addDep : t -> t -> t **)
  
  let addDep i1 i2 =
    match i1 with
    | Bot -> Bot
    | NotBot (l1, u1) ->
      (match i2 with
       | Bot -> Bot
       | NotBot (l2, u2) -> NotBot ((bndAdd l1 l2), (bndAdd u1 u2)))
  
  (** val add : t -> t -> t **)
  
  let add i1 i2 =
    addDep i1 i2
  
  (** val bndNeg : bndT -> bndT **)
  
  let bndNeg = function
  | Infty -> Infty
  | Open n -> Open (ZNum.neg n)
  | Closed n -> Closed (ZNum.neg n)
  
  (** val negDep : t -> t **)
  
  let negDep = function
  | Bot -> Bot
  | NotBot (l, u) -> NotBot ((bndNeg u), (bndNeg l))
  
  (** val neg : t -> t **)
  
  let neg i =
    negDep i
  
  (** val bndMulc : bndT -> ZNum.t -> bndT **)
  
  let bndMulc i n =
    match i with
    | Infty -> Infty
    | Open n' -> Open (ZNum.mul n n')
    | Closed n' -> Closed (ZNum.mul n n')
  
  (** val mulcPos : t -> ZNum.t -> t **)
  
  let mulcPos i0 n0 =
    match i0 with
    | Bot -> Bot
    | NotBot (l, u) -> NotBot ((bndMulc l n0), (bndMulc u n0))
  
  (** val mulcZ : t -> t **)
  
  let mulcZ = function
  | Bot -> Bot
  | NotBot (l, u) ->
    (match l with
     | Infty ->
       (match u with
        | Infty -> NotBot (Infty, Infty)
        | _ -> NotBot (Infty, (Closed ZNum.z)))
     | _ ->
       (match u with
        | Infty -> NotBot ((Closed ZNum.z), Infty)
        | _ -> NotBot ((Closed ZNum.z), (Closed ZNum.z))))
  
  (** val mulcDep : t -> ZNum.t -> t **)
  
  let mulcDep i0 n =
    match ZNum.dec ZNum.z n with
    | Coq_inleft pfn0 ->
      if pfn0 then mulcPos i0 n else negDep (mulcPos i0 (ZNum.neg n))
    | Coq_inright -> mulcZ i0
  
  (** val mulc : t -> ZNum.t -> t **)
  
  let mulc i n =
    mulcDep i n
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Bot -> 'b'::('o'::('t'::[]))
  | NotBot (l, u) ->
    let lstr =
      match l with
      | Infty -> ']'::('-'::('i'::('n'::('f'::('t'::('y'::[]))))))
      | Open n -> append (']'::[]) (ZNum.pr n)
      | Closed n -> append ('['::[]) (ZNum.pr n)
    in
    let ustr =
      match u with
      | Infty -> '+'::('i'::('n'::('f'::('t'::('y'::('['::[]))))))
      | Open n -> append (ZNum.pr n) ('['::[])
      | Closed n -> append (ZNum.pr n) (']'::[])
    in
    append lstr (append (','::(' '::[])) ustr)
  
  (** val tightenL : QItv.bndT -> bndT **)
  
  let tightenL = function
  | QItv.Infty -> Infty
  | QItv.Open q -> Closed (ZtoQ.ceil q)
  | QItv.Closed q -> Closed (ZtoQ.ceil q)
  
  (** val tightenU : QItv.bndT -> bndT **)
  
  let tightenU = function
  | QItv.Infty -> Infty
  | QItv.Open q -> Closed (ZtoQ.floor q)
  | QItv.Closed q -> Closed (ZtoQ.floor q)
  
  (** val fromQItv : QItv.t -> t **)
  
  let fromQItv = function
  | QItv.Bot -> Bot
  | QItv.NotBot (low, up) -> mk (tightenL low) (tightenU up)
 end

