open Datatypes
open FMapPositive
open List0
open NumC
open PositiveMapAddOn
open ProgVar
open String0

module Lin = 
 struct 
  type t = QNum.t PositiveMap.t
  
  (** val absEval : (PVar.t*QNum.t) list -> QNum.t Mem.t -> QNum.t **)
  
  let absEval l m =
    fold_right (fun p sum -> QNum.add sum (QNum.mul (m (fst p)) (snd p)))
      QNum.z l
  
  (** val eval : t -> QNum.t Mem.t -> QNum.t **)
  
  let eval lt m =
    absEval (PositiveMap.elements lt) m
  
  (** val nil : t **)
  
  let nil =
    PositiveMap.Leaf
  
  (** val single : PVar.t -> QNum.t -> t **)
  
  let single x n =
    if QNum.eqDec n QNum.z then nil else single x n
  
  (** val neg : t -> QNum.t PositiveMap.t **)
  
  let neg lt =
    PositiveMap.map (fun c -> QNum.neg c) lt
  
  (** val mul : QNum.t -> t -> t **)
  
  let mul n lt =
    if QNum.eqDec n QNum.z
    then nil
    else PositiveMap.map (fun c -> QNum.mul n c) lt
  
  (** val coq_QNum_eqb : QNum.t -> QNum.t -> bool **)
  
  let coq_QNum_eqb x y =
    if QNum.eqDec x y then true else false
  
  (** val isEq : t -> t -> bool **)
  
  let isEq =
    equal coq_QNum_eqb
  
  (** val add : t -> t -> t **)
  
  let add =
    merge (fun n1 n2 ->
      let r = QNum.add n1 n2 in if QNum.eqDec r QNum.z then None else Some r)
  
  (** val isFree : PVar.t -> t -> bool **)
  
  let isFree x l =
    negb (PositiveMap.mem x l)
  
  (** val rename : PVar.t -> PVar.t -> t -> t **)
  
  let rename x y l =
    match PositiveMap.find x l with
    | Some c -> add (single y c) (PositiveMap.remove x l)
    | None -> l
  
  (** val fmtAux : char list list -> char list -> char list **)
  
  let rec fmtAux l s =
    match l with
    | [] -> s
    | prem::l' -> fmtAux l' (append prem s)
  
  (** val fmt : char list list -> char list **)
  
  let fmt = function
  | [] -> '0'::[]
  | prem::l' -> fmtAux l' prem
  
  (** val pairPr : (PVar.t*QNum.t) -> char list **)
  
  let pairPr p =
    append (PVar.pr (fst p))
      (append (' '::('*'::(' '::[]))) (QNum.pr (snd p)))
  
  (** val pr : t -> char list **)
  
  let pr lt =
    let l = PositiveMap.elements lt in fmt (rev (map pairPr l))
  
  type exportT = (PVar.t*QNum.t) list
  
  (** val export : t -> exportT **)
  
  let export lt =
    PositiveMap.elements lt
  
  (** val import : exportT -> t **)
  
  let import l =
    fold_left (fun lt p -> add (single (fst p) (snd p)) lt) l nil
 end

