open BinInt
open BinNums
open CoqAddOn
open QArith_base
open Qcanon
open Qround
open Specif
open ZArith_dec

module type NumSig = 
 sig 
  type t 
  
  val z : t
  
  val u : t
  
  val eqDec : t -> t -> bool
  
  val ltLeDec : t -> t -> bool
  
  val dec : t -> t -> bool sumor
  
  val add : t -> t -> t
  
  val mul : t -> t -> t
  
  val neg : t -> t
  
  val pr : t -> char list
  
  val prRaw : t -> char list
 end

module QNum = 
 struct 
  type t = coq_Qc
  
  (** val z : t **)
  
  let z =
    coq_Q2Qc { coq_Qnum = Z0; coq_Qden = Coq_xH }
  
  (** val u : t **)
  
  let u =
    coq_Q2Qc { coq_Qnum = (Zpos Coq_xH); coq_Qden = Coq_xH }
  
  (** val add : t -> t -> t **)
  
  let add =
    coq_Qcplus
  
  (** val sub : t -> t -> t **)
  
  let sub =
    coq_Qcminus
  
  (** val mul : t -> t -> t **)
  
  let mul =
    coq_Qcmult
  
  (** val div : t -> t -> t **)
  
  let div =
    coq_Qcdiv
  
  (** val neg : t -> t **)
  
  let neg =
    coq_Qcopp
  
  (** val inv : t -> t **)
  
  let inv =
    coq_Qcinv
  
  (** val eqDec : t -> t -> bool **)
  
  let eqDec =
    coq_Qc_eq_dec
  
  (** val ltLeDec : t -> t -> bool **)
  
  let ltLeDec =
    coq_Qclt_le_dec
  
  (** val dec : t -> t -> bool sumor **)
  
  let dec =
    coq_Qc_dec
  
  (** val pr : t -> char list **)
  
  let pr q =
    let nstr = CoqPr.zPr' (this q).coq_Qnum in
    let dstr = CoqPr.posPr' (this q).coq_Qden in
    sprintf ('('::('%'::('s'::('/'::('%'::('s'::(')'::[])))))))
      (nstr::(dstr::[]))
  
  (** val prRaw : t -> char list **)
  
  let prRaw q =
    let nstr = CoqPr.zPrRaw' (this q).coq_Qnum in
    let dstr = CoqPr.posPrRaw' (this q).coq_Qden in
    sprintf
      ('{'::('n'::('u'::('m'::(' '::('='::(' '::('%'::('s'::(';'::(' '::('d'::('e'::('n'::(' '::('='::(' '::('%'::('s'::('}'::[]))))))))))))))))))))
      (nstr::(dstr::[]))
 end

module ZNum = 
 struct 
  type t = coq_Z
  
  (** val z : t **)
  
  let z =
    Z0
  
  (** val u : t **)
  
  let u =
    Zpos Coq_xH
  
  (** val eqDec : t -> t -> bool **)
  
  let eqDec =
    Z.eq_dec
  
  (** val ltLeDec : t -> t -> bool **)
  
  let ltLeDec =
    coq_Z_lt_le_dec
  
  (** val dec : t -> t -> bool sumor **)
  
  let dec =
    coq_Z_dec'
  
  (** val add : t -> t -> t **)
  
  let add n1 n2 =
    Z.add n1 n2
  
  (** val mul : t -> t -> t **)
  
  let mul n1 n2 =
    Z.mul n1 n2
  
  (** val neg : t -> t **)
  
  let neg n1 =
    Z.opp n1
  
  (** val pr : t -> char list **)
  
  let pr z0 =
    CoqPr.zPr' z0
  
  (** val prRaw : t -> char list **)
  
  let prRaw z0 =
    CoqPr.zPrRaw' z0
 end

module ZtoQ = 
 struct 
  (** val ofZ : ZNum.t -> QNum.t **)
  
  let ofZ z0 =
    inject_Z z0
  
  (** val floor : QNum.t -> ZNum.t **)
  
  let floor n =
    coq_Qfloor (this n)
  
  (** val ceil : QNum.t -> ZNum.t **)
  
  let ceil n =
    coq_Qceiling (this n)
 end

