open AffCond
open AffTerm
open BinNums
open CertC
open ConsSet
open CoqAddOn
open CstrC
open Datatypes
open Debugging
open Itv
open LinTerm
open MSetPositive
open NumC
open ProgVar
open Shadow
open Specif
open String0

module BasicPolQ = 
 struct 
  type polT = { cons : Cs.t; ml : Pol.t }
  
  (** val polT_rect : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1 **)
  
  let polT_rect f p =
    let { cons = x; ml = x0 } = p in f x x0
  
  (** val polT_rec : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1 **)
  
  let polT_rec f p =
    let { cons = x; ml = x0 } = p in f x x0
  
  (** val cons : polT -> Cs.t **)
  
  let cons x = x.cons
  
  (** val ml : polT -> Pol.t **)
  
  let ml x = x.ml
  
  (** val polPr : polT -> char list **)
  
  let polPr p =
    sprintf
      ('{'::('c'::('o'::('q'::(':'::('\\'::('n'::('%'::('s'::('\\'::('n'::('o'::('c'::('a'::('m'::('l'::(':'::('\\'::('n'::('%'::('s'::('}'::[]))))))))))))))))))))))
      ((Cs.pr p.cons)::(((fun p -> CoqPr.stringTr (Pol.prExtRaw p)) p.ml)::[]))
  
  (** val polTop : polT **)
  
  let polTop =
    { cons = Cs.top; ml = Pol.top }
  
  type t = polT option
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Some p' ->
    append ('n'::('o'::('t'::('b'::('o'::('t'::(' '::[]))))))) (polPr p')
  | None -> 'b'::('o'::('t'::[]))
  
  (** val top : t **)
  
  let top =
    Some polTop
  
  (** val bottom : t **)
  
  let bottom =
    None
  
  (** val isIncl : t -> t -> bool **)
  
  let isIncl p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         (match Support.isIncl pol1.ml pol2.ml with
          | NotIncl -> false
          | Incl cert -> Cs.chkIncl pol1.cons pol2.cons cert)
       | None -> false)
    | None -> true
  
  (** val meet : t -> t -> t **)
  
  let meet p1 p2 =
    match p1 with
    | Some p1' ->
      (match p2 with
       | Some p2' ->
         let maybeP,ce = Support.meet p1'.ml p2'.ml in
         (match maybeP with
          | Some p -> Some { cons = (Cs.meet p1'.cons p2'.cons ce); ml = p }
          | None ->
            if Cs.meetEmpty p1'.cons p2'.cons ce
            then None
            else (fun l _ -> failwith (CoqPr.charListTr l))
                   ('P'::('o'::('l'::('C'::('.'::('m'::('e'::('e'::('t'::[])))))))))
                   top)
       | None -> None)
    | None -> None
  
  (** val join : t -> t -> t **)
  
  let join p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         let shadow,cert = Support.join pol1.ml pol2.ml in
         Some { cons = (Cs.buildJoin pol1.cons pol2.cons cert); ml = shadow }
       | None -> p1)
    | None ->
      (match p2 with
       | Some p -> p2
       | None -> None)
  
  (** val widen : t -> t -> t **)
  
  let widen p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         let cs,shadow = Support.widen pol1.ml pol2.ml in
         Some { cons = shadow; ml = cs }
       | None -> p1)
    | None ->
      (match p2 with
       | Some p0 -> p2
       | None -> None)
  
  (** val project : t -> PVar.t -> t **)
  
  let project p x =
    match p with
    | Some p1 ->
      let p2,c = Support.project p1.ml x in
      let cs =
        let nl1,cert1 = Cs.bind p1.cons c in
        (match cert1 with
         | Cert.Implies cert2 -> Cs.build nl1 cert2
         | Cert.Empty f ->
           (fun l _ -> failwith (CoqPr.charListTr l))
             ('C'::('o'::('n'::('s'::('S'::('e'::('t'::('.'::('a'::('p'::('p'::('r'::('o'::('x'::[]))))))))))))))
             Cs.top)
      in
      if Cs.isFree x cs
      then Some { cons = cs; ml = p2 }
      else (fun l _ -> failwith (CoqPr.charListTr l))
             (append
               ('P'::('o'::('l'::('C'::('.'::('p'::('r'::('o'::('j'::('e'::('c'::('t'::(':'::(' '::[]))))))))))))))
               (append (PVar.pr x)
                 (append
                   (' '::('n'::('o'::('t'::(' '::('f'::('r'::('e'::('e'::(' '::('i'::('n'::(' '::[])))))))))))))
                   (Cs.pr cs)))) top
    | None -> None
 end

module PolQItv = 
 struct 
  (** val buildLow : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT **)
  
  let buildLow p v sItv =
    match sItv.low with
    | Infty -> QItv.Infty
    | Open (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.lowerToCstr v n)
      then QItv.Open n
      else QItv.Infty
    | Closed (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.lowerOrEqualsToCstr v n)
      then QItv.Closed n
      else QItv.Infty
  
  (** val buildUp : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT **)
  
  let buildUp p v sItv =
    match sItv.up with
    | Infty -> QItv.Infty
    | Open (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.upperToCstr v n)
      then QItv.Open n
      else QItv.Infty
    | Closed (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.upperOrEqualsToCstr v n)
      then QItv.Closed n
      else QItv.Infty
  
  (** val coq_QItv_mk : QItv.bndT -> QItv.bndT -> QItv.t **)
  
  let coq_QItv_mk l u0 =
    projT1 (sigT_of_sig (QItv.mk l u0))
  
  (** val getItv : BasicPolQ.t -> Lin.t -> QItv.t **)
  
  let getItv p v =
    match p with
    | Some pol0 ->
      let sItv = Support.getItv pol0.BasicPolQ.ml v in
      let low0 = buildLow pol0 v sItv in
      let up0 = buildUp pol0 v sItv in coq_QItv_mk low0 up0
    | None -> QItv.Bot
  
  (** val get_itv : QTerm.t -> BasicPolQ.t -> QItv.t **)
  
  let get_itv a ab =
    let t0,n = QTerm.toLinRec a QNum.u in QItv.shift (getItv ab t0) n
 end

module CstrPol = 
 struct 
  (** val assume : Cstr.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let assume c p =
    match (fun mode l a -> if (Debugging.traceCmp INFO mode) then (print_string (CoqPr.charListTr l); print_newline()); a)
            DEBUG
            ('i'::('n'::(' '::('C'::('s'::('t'::('r'::('P'::('o'::('l'::('.'::('a'::('s'::('s'::('u'::('m'::('e'::[])))))))))))))))))
            p with
    | Some p1 ->
      let opt,cert = Support.add p1.BasicPolQ.ml c in
      (match opt with
       | Some s ->
         Some { BasicPolQ.cons =
           (Cs.redAdd p1.BasicPolQ.cons (Pol.get_nxt p1.BasicPolQ.ml) c cert);
           BasicPolQ.ml = s }
       | None ->
         if Cs.chkEmpty
              (Cs.add p1.BasicPolQ.cons (Pol.get_nxt p1.BasicPolQ.ml) c) cert
         then None
         else (fun l _ -> failwith (CoqPr.charListTr l))
                ('C'::('s'::('t'::('r'::('P'::('o'::('l'::('.'::('a'::('s'::('s'::('u'::('m'::('e'::[]))))))))))))))
                BasicPolQ.top)
    | None -> None
  
  (** val fromCstr : Cstr.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let fromCstr c default =
    match Support.addp Pol.top c with
    | Added (p, p0) ->
      let fwd,bwd = p0 in
      (match Cs.addp Cs.top (Pol.get_nxt Pol.top) c fwd bwd with
       | Some cs -> Some { BasicPolQ.cons = cs; BasicPolQ.ml = p }
       | None ->
         (fun l _ -> failwith (CoqPr.charListTr l))
           ('C'::('s'::('t'::('r'::('P'::('o'::('l'::('.'::('f'::('r'::('o'::('m'::('C'::('s'::('t'::('r'::[]))))))))))))))))
           default)
    | Contrad ce ->
      if Cs.chkEmpty (Cs.add Cs.top (Pol.get_nxt Pol.top) c) ce
      then None
      else (fun l _ -> failwith (CoqPr.charListTr l))
             ('C'::('s'::('t'::('r'::('P'::('o'::('l'::('.'::('f'::('r'::('o'::('m'::('C'::('s'::('t'::('r'::[]))))))))))))))))
             default
  
  (** val require : Cstr.t -> BasicPolQ.t -> bool **)
  
  let require c p =
    BasicPolQ.isIncl p (fromCstr c None)
 end

module NaiveRename = 
 functor (D:sig 
  val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t
 end) ->
 struct 
  (** val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let rename x y a =
    BasicPolQ.project
      (D.assume (QCond.Atom (AffCond.Eq, (Obj.magic (QCond.Term.Var y)),
        (Obj.magic (QCond.Term.Var x)))) a) x
 end

module Rename = 
 functor (D:sig 
  val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t
 end) ->
 struct 
  (** val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let rename x y = function
  | Some p ->
    Some { BasicPolQ.cons = (Cs.rename x y p.BasicPolQ.cons); BasicPolQ.ml =
      (Support.rename x y p.BasicPolQ.ml) }
  | None -> None
 end

module PolLinQ = 
 struct 
  type polT = BasicPolQ.polT = { cons : Cs.t; ml : Pol.t }
  
  (** val polT_rect : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1 **)
  
  let polT_rect f p =
    let { cons = x; ml = x0 } = p in f x x0
  
  (** val polT_rec : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1 **)
  
  let polT_rec f p =
    let { cons = x; ml = x0 } = p in f x x0
  
  (** val cons : polT -> Cs.t **)
  
  let cons p =
    p.cons
  
  (** val ml : polT -> Pol.t **)
  
  let ml p =
    p.ml
  
  (** val polPr : polT -> char list **)
  
  let polPr p =
    sprintf
      ('{'::('c'::('o'::('q'::(':'::('\\'::('n'::('%'::('s'::('\\'::('n'::('o'::('c'::('a'::('m'::('l'::(':'::('\\'::('n'::('%'::('s'::('}'::[]))))))))))))))))))))))
      ((Cs.pr p.cons)::(((fun p -> CoqPr.stringTr (Pol.prExtRaw p)) p.ml)::[]))
  
  (** val polTop : polT **)
  
  let polTop =
    { cons = Cs.top; ml = Pol.top }
  
  type t = polT option
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Some p' ->
    append ('n'::('o'::('t'::('b'::('o'::('t'::(' '::[]))))))) (polPr p')
  | None -> 'b'::('o'::('t'::[]))
  
  (** val top : t **)
  
  let top =
    Some polTop
  
  (** val bottom : t **)
  
  let bottom =
    None
  
  (** val isIncl : t -> t -> bool **)
  
  let isIncl p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         (match Support.isIncl pol1.ml pol2.ml with
          | NotIncl -> false
          | Incl cert -> Cs.chkIncl pol1.cons pol2.cons cert)
       | None -> false)
    | None -> true
  
  (** val meet : t -> t -> t **)
  
  let meet p1 p2 =
    match p1 with
    | Some p1' ->
      (match p2 with
       | Some p2' ->
         let maybeP,ce = Support.meet p1'.ml p2'.ml in
         (match maybeP with
          | Some p -> Some { cons = (Cs.meet p1'.cons p2'.cons ce); ml = p }
          | None ->
            if Cs.meetEmpty p1'.cons p2'.cons ce
            then None
            else (fun l _ -> failwith (CoqPr.charListTr l))
                   ('P'::('o'::('l'::('C'::('.'::('m'::('e'::('e'::('t'::[])))))))))
                   top)
       | None -> None)
    | None -> None
  
  (** val join : t -> t -> t **)
  
  let join p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         let shadow,cert = Support.join pol1.ml pol2.ml in
         Some { cons = (Cs.buildJoin pol1.cons pol2.cons cert); ml = shadow }
       | None -> p1)
    | None ->
      (match p2 with
       | Some p -> p2
       | None -> None)
  
  (** val widen : t -> t -> t **)
  
  let widen p1 p2 =
    match p1 with
    | Some pol1 ->
      (match p2 with
       | Some pol2 ->
         let cs,shadow = Support.widen pol1.ml pol2.ml in
         Some { cons = shadow; ml = cs }
       | None -> p1)
    | None ->
      (match p2 with
       | Some p0 -> p2
       | None -> None)
  
  (** val project : t -> PVar.t -> t **)
  
  let project p x =
    match p with
    | Some p1 ->
      let p2,c = Support.project p1.ml x in
      let cs =
        let nl1,cert1 = Cs.bind p1.cons c in
        (match cert1 with
         | Cert.Implies cert2 -> Cs.build nl1 cert2
         | Cert.Empty f ->
           (fun l _ -> failwith (CoqPr.charListTr l))
             ('C'::('o'::('n'::('s'::('S'::('e'::('t'::('.'::('a'::('p'::('p'::('r'::('o'::('x'::[]))))))))))))))
             Cs.top)
      in
      if Cs.isFree x cs
      then Some { cons = cs; ml = p2 }
      else (fun l _ -> failwith (CoqPr.charListTr l))
             (append
               ('P'::('o'::('l'::('C'::('.'::('p'::('r'::('o'::('j'::('e'::('c'::('t'::(':'::(' '::[]))))))))))))))
               (append (PVar.pr x)
                 (append
                   (' '::('n'::('o'::('t'::(' '::('f'::('r'::('e'::('e'::(' '::('i'::('n'::(' '::[])))))))))))))
                   (Cs.pr cs)))) top
    | None -> None
  
  (** val isBottom : t -> bool **)
  
  let isBottom a =
    isIncl a bottom
  
  (** val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let rec assume c a =
    match c with
    | QCond.Basic b -> if b then a else BasicPolQ.bottom
    | QCond.Atom (cmp, tl, tr) ->
      (match QCond.toCstr tl cmp tr with
       | Some c' -> CstrPol.assume c' a
       | None ->
         (fun l _ -> failwith (CoqPr.charListTr l))
           ('P'::('o'::('l'::('L'::('i'::('n'::('Q'::('.'::('a'::('s'::('s'::('u'::('m'::('e'::(':'::('N'::('e'::('q'::[]))))))))))))))))))
           BasicPolQ.top)
    | QCond.BinL (op, cl, cr) ->
      (match op with
       | AND -> assume cr (assume cl a)
       | OR -> BasicPolQ.join (assume cl a) (assume cr a))
    | QCond.Not c0 ->
      (fun l _ -> failwith (CoqPr.charListTr l))
        ('P'::('o'::('l'::('L'::('i'::('n'::('Q'::('.'::('a'::('s'::('s'::('u'::('m'::('e'::(':'::('N'::('o'::('t'::[]))))))))))))))))))
        BasicPolQ.top
  
  (** val atomicRequire :
      AffCond.cmpT -> QTerm.t -> QTerm.t -> BasicPolQ.t -> bool option **)
  
  let atomicRequire cmp tl tr a =
    match QCond.toCstr tl cmp tr with
    | Some c -> Some (CstrPol.require c a)
    | None -> None
  
  module Naive = 
   struct 
    (** val require : QCond.cond -> t -> bool **)
    
    let require c a =
      isBottom
        (Obj.magic (assume (QCond.nnfForAssume (QCond.Not c)) (Obj.magic a)))
   end
  
  (** val require : QCond.t -> t -> bool **)
  
  let rec require c a =
    match c with
    | QCond.Basic b -> if b then true else isBottom a
    | QCond.Atom (cmp, tl, tr) ->
      (match atomicRequire cmp tl tr (Obj.magic a) with
       | Some b -> b
       | None -> Naive.require c a)
    | QCond.BinL (op, cl, cr) ->
      (match op with
       | AND -> if require cl a then require cr a else false
       | OR -> Naive.require c a)
    | QCond.Not c0 -> Naive.require c a
  
  (** val buildLow : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT **)
  
  let buildLow p v sItv =
    match sItv.low with
    | Infty -> QItv.Infty
    | Open (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.lowerToCstr v n)
      then QItv.Open n
      else QItv.Infty
    | Closed (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.lowerOrEqualsToCstr v n)
      then QItv.Closed n
      else QItv.Infty
  
  (** val buildUp : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT **)
  
  let buildUp p v sItv =
    match sItv.up with
    | Infty -> QItv.Infty
    | Open (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.upperToCstr v n)
      then QItv.Open n
      else QItv.Infty
    | Closed (n, f) ->
      let c = Cs.combine p.BasicPolQ.cons f in
      if Cstr.isIncl c (Cstr.upperOrEqualsToCstr v n)
      then QItv.Closed n
      else QItv.Infty
  
  (** val coq_QItv_mk : QItv.bndT -> QItv.bndT -> QItv.t **)
  
  let coq_QItv_mk l u0 =
    projT1 (sigT_of_sig (QItv.mk l u0))
  
  (** val getItv : BasicPolQ.t -> Lin.t -> QItv.t **)
  
  let getItv p v =
    match p with
    | Some pol0 ->
      let sItv = Support.getItv pol0.BasicPolQ.ml v in
      let low0 = buildLow pol0 v sItv in
      let up0 = buildUp pol0 v sItv in coq_QItv_mk low0 up0
    | None -> QItv.Bot
  
  (** val get_itv : QTerm.t -> BasicPolQ.t -> QItv.t **)
  
  let get_itv a ab =
    let t0,n = QTerm.toLinRec a QNum.u in QItv.shift (getItv ab t0) n
  
  (** val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t **)
  
  let rename x y = function
  | Some p ->
    Some { BasicPolQ.cons = (Cs.rename x y p.BasicPolQ.cons); BasicPolQ.ml =
      (Support.rename x y p.BasicPolQ.ml) }
  | None -> None
 end

module FullPolLinQ = 
 struct 
  (** val encode : bool -> positive -> positive **)
  
  let encode b p =
    if b then Coq_xI p else Coq_xO p
  
  (** val decode : positive -> positive **)
  
  let decode = function
  | Coq_xI p0 -> p0
  | Coq_xO p0 -> p0
  | Coq_xH -> Coq_xH
  
  (** val encodeE : PositiveSet.t -> positive -> positive **)
  
  let encodeE r x =
    encode (PositiveSet.mem x r) x
  
  (** val encodeO : PositiveSet.t -> positive -> positive **)
  
  let encodeO r x =
    encode (negb (PositiveSet.mem x r)) x
  
  (** val decodeIn :
      PositiveSet.t -> QNum.t Mem.t -> QNum.t Mem.t -> positive -> QNum.t **)
  
  let decodeIn r aux m = function
  | Coq_xI p0 -> if PositiveSet.mem p0 r then m p0 else aux (Coq_xI p0)
  | Coq_xO p0 -> if PositiveSet.mem p0 r then aux (Coq_xO p0) else m p0
  | Coq_xH -> aux Coq_xH
  
  (** val switch : PositiveSet.t -> positive -> PositiveSet.t **)
  
  let rec switch m i =
    match m with
    | PositiveSet.Leaf -> PositiveSet.add i PositiveSet.Leaf
    | PositiveSet.Node (l, o, r) ->
      (match i with
       | Coq_xI p -> PositiveSet.node l o (switch r p)
       | Coq_xO p -> PositiveSet.node (switch l p) o r
       | Coq_xH -> PositiveSet.node l (negb o) r)
  
  type assignDomain = { pol : PolLinQ.t; renaming : PositiveSet.t }
  
  (** val assignDomain_rect :
      (PolLinQ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1 **)
  
  let assignDomain_rect f a =
    let { pol = x; renaming = x0 } = a in f x x0
  
  (** val assignDomain_rec :
      (PolLinQ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1 **)
  
  let assignDomain_rec f a =
    let { pol = x; renaming = x0 } = a in f x x0
  
  (** val pol : assignDomain -> PolLinQ.t **)
  
  let pol x = x.pol
  
  (** val renaming : assignDomain -> PositiveSet.t **)
  
  let renaming x = x.renaming
  
  type t = assignDomain
  
  (** val top : t **)
  
  let top =
    { pol = PolLinQ.top; renaming = PositiveSet.empty }
  
  (** val bottom : t **)
  
  let bottom =
    { pol = PolLinQ.bottom; renaming = PositiveSet.empty }
  
  (** val assume : QCond.cond -> t -> t **)
  
  let assume c a =
    { pol =
      (Obj.magic
        (PolLinQ.assume (QCond.map c (encodeE a.renaming)) (Obj.magic a.pol)));
      renaming = a.renaming }
  
  (** val require : QCond.cond -> t -> bool **)
  
  let require c a =
    PolLinQ.require (QCond.map c (encodeE a.renaming)) a.pol
  
  (** val project : t -> PVar.t -> t **)
  
  let project a x =
    { pol = (PolLinQ.project a.pol (encodeE a.renaming x)); renaming =
      a.renaming }
  
  (** val guassignAux : positive -> QCond.cond -> t -> t **)
  
  let guassignAux x c a =
    let tmp = PolLinQ.assume c (Obj.magic a.pol) in
    { pol = (PolLinQ.project (Obj.magic tmp) (encodeE a.renaming x));
    renaming = (switch a.renaming x) }
  
  (** val guassign : PVar.t -> QCond.cond -> t -> t **)
  
  let guassign x c a =
    let f = encodeE a.renaming in
    let c1 = QCond.xmap c f (Mem.assign x (encodeO a.renaming x) f) in
    guassignAux x c1 a
  
  (** val assign : positive -> QCond.Term.term -> t -> t **)
  
  let assign x te a =
    let c = QCond.Atom (AffCond.Eq,
      (Obj.magic (QCond.Term.Var (encodeO a.renaming x))),
      (Obj.magic (QCond.Term.Old (QCond.Term.map te (encodeE a.renaming)))))
    in
    guassignAux x c a
  
  (** val nop : PVar.t -> assignDomain -> t **)
  
  let nop x a =
    { pol =
      (Obj.magic
        (PolLinQ.rename (encodeE a.renaming x) (encodeO a.renaming x)
          (Obj.magic a.pol))); renaming = (switch a.renaming x) }
  
  (** val switchAll : PositiveSet.t -> t -> t **)
  
  let switchAll r a =
    PositiveSet.fold nop r a
  
  (** val isIncl : t -> t -> bool **)
  
  let isIncl a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    PolLinQ.isIncl (switchAll r2 (switchAll r1 a1)).pol a2.pol
  
  (** val join : t -> t -> t **)
  
  let join a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    { pol = (PolLinQ.join (switchAll r1 a1).pol (switchAll r2 a2).pol);
    renaming = (PositiveSet.inter a1.renaming a2.renaming) }
  
  (** val widen : t -> t -> t **)
  
  let widen a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    { pol = (PolLinQ.widen (switchAll r1 a1).pol (switchAll r2 a2).pol);
    renaming = (PositiveSet.inter a1.renaming a2.renaming) }
  
  (** val isBottom : t -> bool **)
  
  let isBottom a =
    isIncl a bottom
  
  (** val get_itv : QCond.Term.t -> t -> QItv.t **)
  
  let get_itv te a =
    PolQItv.get_itv (Obj.magic (QCond.Term.map te (encodeE a.renaming)))
      (Obj.magic a.pol)
  
  (** val pr : t -> char list **)
  
  let pr a =
    PolLinQ.pr a.pol
 end

