open AffCond
open AffTerm
open BinNums
open CertC
open ConsSet
open CoqAddOn
open CstrC
open Datatypes
open Debugging
open Itv
open LinTerm
open MSetPositive
open NumC
open ProgVar
open Shadow
open Specif
open String0

module BasicPolQ : 
 sig 
  type polT = { cons : Cs.t; ml : Pol.t }
  
  val polT_rect : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1
  
  val polT_rec : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1
  
  val cons : polT -> Cs.t
  
  val ml : polT -> Pol.t
  
  val polPr : polT -> char list
  
  val polTop : polT
  
  type t = polT option
  
  val pr : t -> char list
  
  val top : t
  
  val bottom : t
  
  val isIncl : t -> t -> bool
  
  val meet : t -> t -> t
  
  val join : t -> t -> t
  
  val widen : t -> t -> t
  
  val project : t -> PVar.t -> t
 end

module PolQItv : 
 sig 
  val buildLow : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT
  
  val buildUp : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT
  
  val coq_QItv_mk : QItv.bndT -> QItv.bndT -> QItv.t
  
  val getItv : BasicPolQ.t -> Lin.t -> QItv.t
  
  val get_itv : QTerm.t -> BasicPolQ.t -> QItv.t
 end

module CstrPol : 
 sig 
  val assume : Cstr.t -> BasicPolQ.t -> BasicPolQ.t
  
  val fromCstr : Cstr.t -> BasicPolQ.t -> BasicPolQ.t
  
  val require : Cstr.t -> BasicPolQ.t -> bool
 end

module NaiveRename : 
 functor (D:sig 
  val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t
 end) ->
 sig 
  val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t
 end

module Rename : 
 functor (D:sig 
  val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t
 end) ->
 sig 
  val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t
 end

module PolLinQ : 
 sig 
  type polT = BasicPolQ.polT = { cons : Cs.t; ml : Pol.t }
  
  val polT_rect : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1
  
  val polT_rec : (Cs.t -> Pol.t -> 'a1) -> polT -> 'a1
  
  val cons : polT -> Cs.t
  
  val ml : polT -> Pol.t
  
  val polPr : polT -> char list
  
  val polTop : polT
  
  type t = polT option
  
  val pr : t -> char list
  
  val top : t
  
  val bottom : t
  
  val isIncl : t -> t -> bool
  
  val meet : t -> t -> t
  
  val join : t -> t -> t
  
  val widen : t -> t -> t
  
  val project : t -> PVar.t -> t
  
  val isBottom : t -> bool
  
  val assume : QCond.t -> BasicPolQ.t -> BasicPolQ.t
  
  val atomicRequire :
    AffCond.cmpT -> QTerm.t -> QTerm.t -> BasicPolQ.t -> bool option
  
  module Naive : 
   sig 
    val require : QCond.cond -> t -> bool
   end
  
  val require : QCond.t -> t -> bool
  
  val buildLow : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT
  
  val buildUp : BasicPolQ.polT -> Lin.t -> itvT -> QItv.bndT
  
  val coq_QItv_mk : QItv.bndT -> QItv.bndT -> QItv.t
  
  val getItv : BasicPolQ.t -> Lin.t -> QItv.t
  
  val get_itv : QTerm.t -> BasicPolQ.t -> QItv.t
  
  val rename : PVar.t -> PVar.t -> BasicPolQ.t -> BasicPolQ.t
 end

module FullPolLinQ : 
 sig 
  val encode : bool -> positive -> positive
  
  val decode : positive -> positive
  
  val encodeE : PositiveSet.t -> positive -> positive
  
  val encodeO : PositiveSet.t -> positive -> positive
  
  val decodeIn :
    PositiveSet.t -> QNum.t Mem.t -> QNum.t Mem.t -> positive -> QNum.t
  
  val switch : PositiveSet.t -> positive -> PositiveSet.t
  
  type assignDomain = { pol : PolLinQ.t; renaming : PositiveSet.t }
  
  val assignDomain_rect :
    (PolLinQ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1
  
  val assignDomain_rec :
    (PolLinQ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1
  
  val pol : assignDomain -> PolLinQ.t
  
  val renaming : assignDomain -> PositiveSet.t
  
  type t = assignDomain
  
  val top : t
  
  val bottom : t
  
  val assume : QCond.cond -> t -> t
  
  val require : QCond.cond -> t -> bool
  
  val project : t -> PVar.t -> t
  
  val guassignAux : positive -> QCond.cond -> t -> t
  
  val guassign : PVar.t -> QCond.cond -> t -> t
  
  val assign : positive -> QCond.Term.term -> t -> t
  
  val nop : PVar.t -> assignDomain -> t
  
  val switchAll : PositiveSet.t -> t -> t
  
  val isIncl : t -> t -> bool
  
  val join : t -> t -> t
  
  val widen : t -> t -> t
  
  val isBottom : t -> bool
  
  val get_itv : QCond.Term.t -> t -> QItv.t
  
  val pr : t -> char list
 end

