open AffCond
open AffTerm
open BinNums
open Datatypes
open Itv
open MSetPositive
open NumC
open PolC
open ProgVar

module CorePolLinZ = 
 struct 
  type t = PolLinQ.t
  
  (** val isIncl : PolLinQ.t -> PolLinQ.t -> bool **)
  
  let isIncl =
    PolLinQ.isIncl
  
  (** val top : PolLinQ.t **)
  
  let top =
    PolLinQ.top
  
  (** val bottom : t **)
  
  let bottom =
    PolLinQ.bottom
  
  (** val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let widen =
    PolLinQ.widen
  
  (** val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let join =
    PolLinQ.join
  
  (** val assume : ZCond.t -> t -> t **)
  
  let assume c a =
    Obj.magic (PolLinQ.assume (QCond.lift c) (Obj.magic a))
  
  (** val project : t -> PVar.t -> t **)
  
  let project a x =
    PolLinQ.project a x
  
  (** val rename : PVar.t -> PVar.t -> t -> t **)
  
  let rename x y a =
    Obj.magic (PolLinQ.rename x y (Obj.magic a))
  
  (** val pr : t -> char list **)
  
  let pr ab =
    PolLinQ.pr ab
 end

module PolZItv = 
 struct 
  (** val get_itv : ZTerm.t -> CorePolLinZ.t -> ZItv.t **)
  
  let get_itv a ab =
    let qa = QTerm.lift a in
    let qitv = PolLinQ.get_itv qa (Obj.magic ab) in ZItv.fromQItv qitv
 end

module WeakAbsPolLinZ = 
 struct 
  type t = PolLinQ.t
  
  (** val isIncl : PolLinQ.t -> PolLinQ.t -> bool **)
  
  let isIncl =
    PolLinQ.isIncl
  
  (** val top : PolLinQ.t **)
  
  let top =
    PolLinQ.top
  
  (** val bottom : t **)
  
  let bottom =
    PolLinQ.bottom
  
  (** val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let widen =
    PolLinQ.widen
  
  (** val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let join =
    PolLinQ.join
  
  (** val assume : ZCond.t -> t -> t **)
  
  let assume c a =
    Obj.magic (PolLinQ.assume (QCond.lift c) (Obj.magic a))
  
  (** val project : t -> PVar.t -> t **)
  
  let project a x =
    PolLinQ.project a x
  
  (** val rename : PVar.t -> PVar.t -> t -> t **)
  
  let rename x y a =
    Obj.magic (PolLinQ.rename x y (Obj.magic a))
  
  (** val pr : t -> char list **)
  
  let pr ab =
    PolLinQ.pr ab
  
  (** val isBottom : t -> bool **)
  
  let isBottom a =
    isIncl a bottom
 end

module ZAtomicRequire = 
 struct 
  (** val atomicRequire :
      cmpT -> ZTerm.t -> ZTerm.t -> WeakAbsPolLinZ.t -> bool option **)
  
  let atomicRequire cmp tl tr a =
    PolLinQ.atomicRequire cmp (QTerm.lift tl) (QTerm.lift tr) (Obj.magic a)
 end

module PolLinZ = 
 struct 
  type t = PolLinQ.t
  
  (** val isIncl : PolLinQ.t -> PolLinQ.t -> bool **)
  
  let isIncl =
    PolLinQ.isIncl
  
  (** val top : PolLinQ.t **)
  
  let top =
    PolLinQ.top
  
  (** val bottom : t **)
  
  let bottom =
    PolLinQ.bottom
  
  (** val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let widen =
    PolLinQ.widen
  
  (** val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t **)
  
  let join =
    PolLinQ.join
  
  (** val assume : ZCond.t -> t -> t **)
  
  let assume c a =
    Obj.magic (PolLinQ.assume (QCond.lift c) (Obj.magic a))
  
  (** val project : t -> PVar.t -> t **)
  
  let project a x =
    PolLinQ.project a x
  
  (** val rename : PVar.t -> PVar.t -> t -> t **)
  
  let rename x y a =
    Obj.magic (PolLinQ.rename x y (Obj.magic a))
  
  (** val pr : t -> char list **)
  
  let pr ab =
    PolLinQ.pr ab
  
  (** val isBottom : t -> bool **)
  
  let isBottom a =
    isIncl a bottom
  
  (** val atomicRequire :
      cmpT -> ZTerm.t -> ZTerm.t -> WeakAbsPolLinZ.t -> bool option **)
  
  let atomicRequire cmp tl tr a =
    PolLinQ.atomicRequire cmp (QTerm.lift tl) (QTerm.lift tr) (Obj.magic a)
  
  module Naive = 
   struct 
    (** val require : ZCond.cond -> t -> bool **)
    
    let require c a =
      isBottom (assume (ZCond.nnfForAssume (ZCond.Not c)) a)
   end
  
  (** val require : ZCond.t -> t -> bool **)
  
  let rec require c a =
    match c with
    | ZCond.Basic b -> if b then true else isBottom a
    | ZCond.Atom (cmp, tl, tr) ->
      (match atomicRequire cmp tl tr a with
       | Some b -> b
       | None -> Naive.require c a)
    | ZCond.BinL (op, cl, cr) ->
      (match op with
       | AND -> if require cl a then require cr a else false
       | OR -> Naive.require c a)
    | ZCond.Not c0 -> Naive.require c a
  
  (** val get_itv : ZTerm.t -> CorePolLinZ.t -> ZItv.t **)
  
  let get_itv a ab =
    let qa = QTerm.lift a in
    let qitv = PolLinQ.get_itv qa (Obj.magic ab) in ZItv.fromQItv qitv
 end

module FullPolLinZ = 
 struct 
  (** val encode : bool -> positive -> positive **)
  
  let encode b p =
    if b then Coq_xI p else Coq_xO p
  
  (** val decode : positive -> positive **)
  
  let decode = function
  | Coq_xI p0 -> p0
  | Coq_xO p0 -> p0
  | Coq_xH -> Coq_xH
  
  (** val encodeE : PositiveSet.t -> positive -> positive **)
  
  let encodeE r x =
    encode (PositiveSet.mem x r) x
  
  (** val encodeO : PositiveSet.t -> positive -> positive **)
  
  let encodeO r x =
    encode (negb (PositiveSet.mem x r)) x
  
  (** val decodeIn :
      PositiveSet.t -> ZNum.t Mem.t -> ZNum.t Mem.t -> positive -> ZNum.t **)
  
  let decodeIn r aux m = function
  | Coq_xI p0 -> if PositiveSet.mem p0 r then m p0 else aux (Coq_xI p0)
  | Coq_xO p0 -> if PositiveSet.mem p0 r then aux (Coq_xO p0) else m p0
  | Coq_xH -> aux Coq_xH
  
  (** val switch : PositiveSet.t -> positive -> PositiveSet.t **)
  
  let rec switch m i =
    match m with
    | PositiveSet.Leaf -> PositiveSet.add i PositiveSet.Leaf
    | PositiveSet.Node (l, o, r) ->
      (match i with
       | Coq_xI p -> PositiveSet.node l o (switch r p)
       | Coq_xO p -> PositiveSet.node (switch l p) o r
       | Coq_xH -> PositiveSet.node l (negb o) r)
  
  type assignDomain = { pol : PolLinZ.t; renaming : PositiveSet.t }
  
  (** val assignDomain_rect :
      (PolLinZ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1 **)
  
  let assignDomain_rect f a =
    let { pol = x; renaming = x0 } = a in f x x0
  
  (** val assignDomain_rec :
      (PolLinZ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1 **)
  
  let assignDomain_rec f a =
    let { pol = x; renaming = x0 } = a in f x x0
  
  (** val pol : assignDomain -> PolLinZ.t **)
  
  let pol x = x.pol
  
  (** val renaming : assignDomain -> PositiveSet.t **)
  
  let renaming x = x.renaming
  
  type t = assignDomain
  
  (** val top : t **)
  
  let top =
    { pol = PolLinZ.top; renaming = PositiveSet.empty }
  
  (** val bottom : t **)
  
  let bottom =
    { pol = PolLinZ.bottom; renaming = PositiveSet.empty }
  
  (** val assume : ZCond.cond -> t -> t **)
  
  let assume c a =
    { pol = (PolLinZ.assume (ZCond.map c (encodeE a.renaming)) a.pol);
      renaming = a.renaming }
  
  (** val require : ZCond.cond -> t -> bool **)
  
  let require c a =
    PolLinZ.require (ZCond.map c (encodeE a.renaming)) a.pol
  
  (** val project : t -> PVar.t -> t **)
  
  let project a x =
    { pol = (PolLinZ.project a.pol (encodeE a.renaming x)); renaming =
      a.renaming }
  
  (** val guassignAux : positive -> ZCond.cond -> t -> t **)
  
  let guassignAux x c a =
    let tmp = PolLinZ.assume c a.pol in
    { pol = (PolLinZ.project tmp (encodeE a.renaming x)); renaming =
    (switch a.renaming x) }
  
  (** val guassign : PVar.t -> ZCond.cond -> t -> t **)
  
  let guassign x c a =
    let f = encodeE a.renaming in
    let c1 = ZCond.xmap c f (Mem.assign x (encodeO a.renaming x) f) in
    guassignAux x c1 a
  
  (** val assign : positive -> ZCond.Term.term -> t -> t **)
  
  let assign x te a =
    let c = ZCond.Atom (AffCond.Eq,
      (Obj.magic (ZCond.Term.Var (encodeO a.renaming x))),
      (Obj.magic (ZCond.Term.Old (ZCond.Term.map te (encodeE a.renaming)))))
    in
    guassignAux x c a
  
  (** val nop : PVar.t -> assignDomain -> t **)
  
  let nop x a =
    { pol =
      (PolLinZ.rename (encodeE a.renaming x) (encodeO a.renaming x) a.pol);
      renaming = (switch a.renaming x) }
  
  (** val switchAll : PositiveSet.t -> t -> t **)
  
  let switchAll r a =
    PositiveSet.fold nop r a
  
  (** val isIncl : t -> t -> bool **)
  
  let isIncl a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    PolLinZ.isIncl (switchAll r2 (switchAll r1 a1)).pol a2.pol
  
  (** val join : t -> t -> t **)
  
  let join a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    { pol = (PolLinZ.join (switchAll r1 a1).pol (switchAll r2 a2).pol);
    renaming = (PositiveSet.inter a1.renaming a2.renaming) }
  
  (** val widen : t -> t -> t **)
  
  let widen a1 a2 =
    let r1 = PositiveSet.diff a1.renaming a2.renaming in
    let r2 = PositiveSet.diff a2.renaming a1.renaming in
    { pol = (PolLinZ.widen (switchAll r1 a1).pol (switchAll r2 a2).pol);
    renaming = (PositiveSet.inter a1.renaming a2.renaming) }
  
  (** val isBottom : t -> bool **)
  
  let isBottom a =
    isIncl a bottom
  
  (** val get_itv : ZCond.Term.t -> t -> ZItv.t **)
  
  let get_itv te a =
    PolZItv.get_itv (Obj.magic (ZCond.Term.map te (encodeE a.renaming)))
      a.pol
  
  (** val pr : t -> char list **)
  
  let pr a =
    PolLinZ.pr a.pol
 end

