open AffCond
open AffTerm
open BinNums
open Datatypes
open Itv
open MSetPositive
open NumC
open PolC
open ProgVar

module CorePolLinZ : 
 sig 
  type t = PolLinQ.t
  
  val isIncl : PolLinQ.t -> PolLinQ.t -> bool
  
  val top : PolLinQ.t
  
  val bottom : t
  
  val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val assume : ZCond.t -> t -> t
  
  val project : t -> PVar.t -> t
  
  val rename : PVar.t -> PVar.t -> t -> t
  
  val pr : t -> char list
 end

module PolZItv : 
 sig 
  val get_itv : ZTerm.t -> CorePolLinZ.t -> ZItv.t
 end

module WeakAbsPolLinZ : 
 sig 
  type t = PolLinQ.t
  
  val isIncl : PolLinQ.t -> PolLinQ.t -> bool
  
  val top : PolLinQ.t
  
  val bottom : t
  
  val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val assume : ZCond.t -> t -> t
  
  val project : t -> PVar.t -> t
  
  val rename : PVar.t -> PVar.t -> t -> t
  
  val pr : t -> char list
  
  val isBottom : t -> bool
 end

module ZAtomicRequire : 
 sig 
  val atomicRequire :
    cmpT -> ZTerm.t -> ZTerm.t -> WeakAbsPolLinZ.t -> bool option
 end

module PolLinZ : 
 sig 
  type t = PolLinQ.t
  
  val isIncl : PolLinQ.t -> PolLinQ.t -> bool
  
  val top : PolLinQ.t
  
  val bottom : t
  
  val widen : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val join : PolLinQ.t -> PolLinQ.t -> PolLinQ.t
  
  val assume : ZCond.t -> t -> t
  
  val project : t -> PVar.t -> t
  
  val rename : PVar.t -> PVar.t -> t -> t
  
  val pr : t -> char list
  
  val isBottom : t -> bool
  
  val atomicRequire :
    cmpT -> ZTerm.t -> ZTerm.t -> WeakAbsPolLinZ.t -> bool option
  
  module Naive : 
   sig 
    val require : ZCond.cond -> t -> bool
   end
  
  val require : ZCond.t -> t -> bool
  
  val get_itv : ZTerm.t -> CorePolLinZ.t -> ZItv.t
 end

module FullPolLinZ : 
 sig 
  val encode : bool -> positive -> positive
  
  val decode : positive -> positive
  
  val encodeE : PositiveSet.t -> positive -> positive
  
  val encodeO : PositiveSet.t -> positive -> positive
  
  val decodeIn :
    PositiveSet.t -> ZNum.t Mem.t -> ZNum.t Mem.t -> positive -> ZNum.t
  
  val switch : PositiveSet.t -> positive -> PositiveSet.t
  
  type assignDomain = { pol : PolLinZ.t; renaming : PositiveSet.t }
  
  val assignDomain_rect :
    (PolLinZ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1
  
  val assignDomain_rec :
    (PolLinZ.t -> PositiveSet.t -> 'a1) -> assignDomain -> 'a1
  
  val pol : assignDomain -> PolLinZ.t
  
  val renaming : assignDomain -> PositiveSet.t
  
  type t = assignDomain
  
  val top : t
  
  val bottom : t
  
  val assume : ZCond.cond -> t -> t
  
  val require : ZCond.cond -> t -> bool
  
  val project : t -> PVar.t -> t
  
  val guassignAux : positive -> ZCond.cond -> t -> t
  
  val guassign : PVar.t -> ZCond.cond -> t -> t
  
  val assign : positive -> ZCond.Term.term -> t -> t
  
  val nop : PVar.t -> assignDomain -> t
  
  val switchAll : PositiveSet.t -> t -> t
  
  val isIncl : t -> t -> bool
  
  val join : t -> t -> t
  
  val widen : t -> t -> t
  
  val isBottom : t -> bool
  
  val get_itv : ZCond.Term.t -> t -> ZItv.t
  
  val pr : t -> char list
 end

