open AffCond
open AffTerm
open BinNums
open CoqAddOn
open Itv
open NumC
open ProgVar

module Cmp = 
 struct 
  type t =
  | Eq
  | Le
  | Lt
  | Neq
  | Ge
  | Gt
  
  (** val t_rect : 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> t -> 'a1 **)
  
  let t_rect f f0 f1 f2 f3 f4 = function
  | Eq -> f
  | Le -> f0
  | Lt -> f1
  | Neq -> f2
  | Ge -> f3
  | Gt -> f4
  
  (** val t_rec : 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> t -> 'a1 **)
  
  let t_rec f f0 f1 f2 f3 f4 = function
  | Eq -> f
  | Le -> f0
  | Lt -> f1
  | Neq -> f2
  | Ge -> f3
  | Gt -> f4
  
  (** val pr : t -> char list **)
  
  let pr = function
  | Eq -> '='::[]
  | Le -> '<'::('='::[])
  | Lt -> '<'::[]
  | Neq -> '/'::('='::[])
  | Ge -> '>'::('='::[])
  | Gt -> '>'::[]
  
  (** val dec : t -> ZNum.t -> ZNum.t -> bool **)
  
  let dec c z z' =
    match c with
    | Eq -> ZNum.eqDec z z'
    | Le -> if ZNum.ltLeDec z' z then false else true
    | Lt -> ZNum.ltLeDec z z'
    | Neq -> if ZNum.eqDec z z' then false else true
    | Ge -> if ZNum.ltLeDec z z' then false else true
    | Gt -> ZNum.ltLeDec z' z
 end

module Expr = 
 struct 
  type binopT =
  | Oadd
  | Osub
  | Omul
  | Odiv
  | Omod
  | Oand
  | Oor
  | Oxor
  | Oshl
  | Oshr
  | Ocmp of Cmp.t
  
  (** val binopT_rect :
      'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 ->
      (Cmp.t -> 'a1) -> binopT -> 'a1 **)
  
  let binopT_rect f f0 f1 f2 f3 f4 f5 f6 f7 f8 f9 = function
  | Oadd -> f
  | Osub -> f0
  | Omul -> f1
  | Odiv -> f2
  | Omod -> f3
  | Oand -> f4
  | Oor -> f5
  | Oxor -> f6
  | Oshl -> f7
  | Oshr -> f8
  | Ocmp x -> f9 x
  
  (** val binopT_rec :
      'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 ->
      (Cmp.t -> 'a1) -> binopT -> 'a1 **)
  
  let binopT_rec f f0 f1 f2 f3 f4 f5 f6 f7 f8 f9 = function
  | Oadd -> f
  | Osub -> f0
  | Omul -> f1
  | Odiv -> f2
  | Omod -> f3
  | Oand -> f4
  | Oor -> f5
  | Oxor -> f6
  | Oshl -> f7
  | Oshr -> f8
  | Ocmp x -> f9 x
  
  type unopT =
  | Oneg
  | Onot
  
  (** val unopT_rect : 'a1 -> 'a1 -> unopT -> 'a1 **)
  
  let unopT_rect f f0 = function
  | Oneg -> f
  | Onot -> f0
  
  (** val unopT_rec : 'a1 -> 'a1 -> unopT -> 'a1 **)
  
  let unopT_rec f f0 = function
  | Oneg -> f
  | Onot -> f0
  
  type t =
  | Evar of PVar.t
  | Econst of ZNum.t
  | Eitv of ZItv.t
  | Eunknown
  | Eunop of unopT * t
  | Ebinop of binopT * t * t
  
  (** val t_rect :
      (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (ZItv.t -> 'a1) -> 'a1 -> (unopT
      -> t -> 'a1 -> 'a1) -> (binopT -> t -> 'a1 -> t -> 'a1 -> 'a1) -> t ->
      'a1 **)
  
  let rec t_rect f f0 f1 f2 f3 f4 = function
  | Evar t1 -> f t1
  | Econst t1 -> f0 t1
  | Eitv t1 -> f1 t1
  | Eunknown -> f2
  | Eunop (u, t1) -> f3 u t1 (t_rect f f0 f1 f2 f3 f4 t1)
  | Ebinop (b, t1, t2) ->
    f4 b t1 (t_rect f f0 f1 f2 f3 f4 t1) t2 (t_rect f f0 f1 f2 f3 f4 t2)
  
  (** val t_rec :
      (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (ZItv.t -> 'a1) -> 'a1 -> (unopT
      -> t -> 'a1 -> 'a1) -> (binopT -> t -> 'a1 -> t -> 'a1 -> 'a1) -> t ->
      'a1 **)
  
  let rec t_rec f f0 f1 f2 f3 f4 = function
  | Evar t1 -> f t1
  | Econst t1 -> f0 t1
  | Eitv t1 -> f1 t1
  | Eunknown -> f2
  | Eunop (u, t1) -> f3 u t1 (t_rec f f0 f1 f2 f3 f4 t1)
  | Ebinop (b, t1, t2) ->
    f4 b t1 (t_rec f f0 f1 f2 f3 f4 t1) t2 (t_rec f f0 f1 f2 f3 f4 t2)
  
  (** val pr : t -> char list **)
  
  let rec pr = function
  | Evar x ->
    sprintf ('E'::('v'::('a'::('r'::(' '::('('::('%'::('s'::(')'::[])))))))))
      ((PVar.pr x)::[])
  | Econst n ->
    sprintf
      ('E'::('c'::('o'::('n'::('s'::('t'::(' '::('('::('%'::('s'::(')'::[])))))))))))
      ((ZNum.pr n)::[])
  | Eitv i ->
    sprintf ('E'::('i'::('t'::('v'::(' '::('('::('%'::('s'::(')'::[])))))))))
      ((ZItv.pr i)::[])
  | Eunknown -> 'u'::('n'::('k'::('n'::('o'::('w'::('n'::[]))))))
  | Eunop (op, e0) ->
    let estr = pr e0 in
    (match op with
     | Oneg -> sprintf ('-'::('('::('%'::('s'::(')'::[]))))) (estr::[])
     | Onot ->
       sprintf ('n'::('o'::('t'::('('::('%'::('s'::(')'::[]))))))) (estr::[]))
  | Ebinop (op, e1, e2) ->
    let ostr =
      match op with
      | Oadd -> '+'::[]
      | Osub -> '-'::[]
      | Omul -> '*'::[]
      | Odiv -> '/'::[]
      | Omod -> 'm'::('o'::('d'::[]))
      | Oand -> '&'::[]
      | Oor -> '|'::[]
      | Oxor -> 'x'::('o'::('r'::[]))
      | Oshl -> 's'::('h'::('l'::[]))
      | Oshr -> 'o'::('s'::('h'::('r'::[])))
      | Ocmp c -> Cmp.pr c
    in
    sprintf
      ('('::('%'::('s'::(')'::(' '::('%'::('s'::(' '::('('::('%'::('s'::(')'::[]))))))))))))
      ((pr e1)::(ostr::((pr e2)::[])))
 end

module Tr = 
 struct 
  (** val itv2z : Expr.t -> Expr.t **)
  
  let rec itv2z e = match e with
  | Expr.Eitv i ->
    (match i with
     | ZItv.Bot -> Expr.Eitv i
     | ZItv.NotBot (low, up) ->
       (match low with
        | ZItv.Closed l ->
          (match up with
           | ZItv.Closed u -> if ZNum.eqDec l u then Expr.Econst l else e
           | _ -> Expr.Eitv i)
        | _ -> Expr.Eitv i))
  | Expr.Eunop (op, e') -> Expr.Eunop (op, (itv2z e'))
  | Expr.Ebinop (op, e1, e2) -> Expr.Ebinop (op, (itv2z e1), (itv2z e2))
  | x -> x
  
  (** val sub2add : Expr.t -> Expr.t **)
  
  let rec sub2add e = match e with
  | Expr.Eunop (op, e') -> Expr.Eunop (op, (sub2add e'))
  | Expr.Ebinop (b, e1, e2) ->
    (match b with
     | Expr.Osub ->
       Expr.Ebinop (Expr.Oadd, (sub2add e1), (Expr.Eunop (Expr.Oneg,
         (sub2add e2))))
     | _ -> e)
  | _ -> e
  
  (** val simplify : Expr.t -> Expr.t **)
  
  let simplify e =
    sub2add (itv2z e)
  
  (** val bind : 'a1 option -> ('a1 -> 'a2 option) -> 'a2 option **)
  
  let bind maybe f =
    match maybe with
    | Some e -> f e
    | None -> None
  
  (** val expr_aux : Expr.t -> ZTerm.t option **)
  
  let rec expr_aux = function
  | Expr.Evar x -> Some (ZTerm.Var x)
  | Expr.Econst z -> Some (ZTerm.Cte z)
  | Expr.Eunop (op, e') ->
    (match op with
     | Expr.Oneg -> bind (expr_aux e') (fun e0 -> Some (ZTerm.Opp e0))
     | Expr.Onot -> None)
  | Expr.Ebinop (op, e1, e2) ->
    (match op with
     | Expr.Oadd ->
       bind (expr_aux e1) (fun e3 ->
         bind (expr_aux e2) (fun e4 -> Some (ZTerm.Plus (e3, e4))))
     | Expr.Omul ->
       (match e1 with
        | Expr.Econst z ->
          bind (expr_aux e2) (fun e0 -> Some (ZTerm.Mult (e0, z)))
        | _ ->
          (match e2 with
           | Expr.Econst z ->
             bind (expr_aux e1) (fun e0 -> Some (ZTerm.Mult (e0, z)))
           | _ -> None))
     | _ -> None)
  | _ -> None
  
  (** val expr : Expr.t -> ZTerm.t option **)
  
  let expr e =
    expr_aux (simplify e)
  
  (** val cmp : Cmp.t -> ZTerm.t -> ZTerm.t -> ZCond.t **)
  
  let cmp r e1 e2 =
    match r with
    | Cmp.Eq -> ZCond.Atom (Eq, e1, e2)
    | Cmp.Le -> ZCond.Atom (Le, e1, e2)
    | Cmp.Lt -> ZCond.Atom (Lt, e1, e2)
    | Cmp.Neq -> ZCond.Atom (Neq, e1, e2)
    | Cmp.Ge -> ZCond.Atom (Le, e2, e1)
    | Cmp.Gt -> ZCond.Atom (Lt, e2, e1)
  
  (** val cond : Expr.t -> ZCond.t option **)
  
  let rec cond = function
  | Expr.Ebinop (b, e1, e2) ->
    (match b with
     | Expr.Ocmp r ->
       bind (expr e1) (fun e1' ->
         bind (expr e2) (fun e2' -> Some (cmp r e1' e2')))
     | _ -> None)
  | _ -> None
  
  (** val asg_itv : PVar.t -> ZItv.t -> ZCond.t option **)
  
  let asg_itv x = function
  | ZItv.Bot -> None
  | ZItv.NotBot (lbnd, ubnd) ->
    let mk = fun l' ->
      match ubnd with
      | ZItv.Infty -> Some l'
      | ZItv.Open u ->
        let u' = ZCond.Atom (Lt, (ZTerm.Var x), (ZTerm.Cte u)) in
        Some (ZCond.BinL (AND, l', u'))
      | ZItv.Closed u ->
        let u' = ZCond.Atom (Le, (ZTerm.Var x), (ZTerm.Cte u)) in
        Some (ZCond.BinL (AND, l', u'))
    in
    (match lbnd with
     | ZItv.Infty ->
       (match ubnd with
        | ZItv.Infty -> None
        | ZItv.Open u -> Some (ZCond.Atom (Lt, (ZTerm.Var x), (ZTerm.Cte u)))
        | ZItv.Closed u ->
          Some (ZCond.Atom (Le, (ZTerm.Var x), (ZTerm.Cte u))))
     | ZItv.Open l -> mk (ZCond.Atom (Lt, (ZTerm.Cte l), (ZTerm.Var x)))
     | ZItv.Closed l -> mk (ZCond.Atom (Le, (ZTerm.Cte l), (ZTerm.Var x))))
  
  (** val asg : PVar.t -> Expr.t -> ZCond.t option **)
  
  let asg x e =
    match simplify e with
    | Expr.Eitv i -> asg_itv x i
    | x0 ->
      (match expr_aux x0 with
       | Some e' -> Some (ZCond.Atom (Eq, (ZTerm.Var x), (ZTerm.Old e')))
       | None -> None)
 end

module AbDom = 
 struct 
  type coq_A = PolLinZ.FullPolLinZ.t
  
  (** val pr : coq_A -> char list **)
  
  let pr =
    PolLinZ.FullPolLinZ.pr
  
  type coq_B = ZNum.t Mem.t
  
  (** val top : coq_A **)
  
  let top =
    PolLinZ.FullPolLinZ.top
  
  (** val leb : coq_A -> coq_A -> bool **)
  
  let leb ab1 ab2 =
    PolLinZ.FullPolLinZ.isIncl ab1 ab2
  
  (** val bottom : coq_A **)
  
  let bottom =
    PolLinZ.FullPolLinZ.bottom
  
  (** val isBottom : coq_A -> bool **)
  
  let isBottom =
    PolLinZ.FullPolLinZ.isBottom
  
  (** val join : coq_A -> coq_A -> coq_A **)
  
  let join ab1 ab2 =
    PolLinZ.FullPolLinZ.join ab1 ab2
  
  (** val widen : coq_A -> coq_A -> coq_A **)
  
  let widen =
    PolLinZ.FullPolLinZ.widen
  
  (** val itv_one : ZItv.t **)
  
  let itv_one =
    ZItv.NotBot ((ZItv.Closed (Zpos Coq_xH)), (ZItv.Closed (Zpos Coq_xH)))
  
  (** val itv_zero : ZItv.t **)
  
  let itv_zero =
    ZItv.NotBot ((ZItv.Closed Z0), (ZItv.Closed Z0))
  
  (** val itv_unit : ZItv.t **)
  
  let itv_unit =
    ZItv.NotBot ((ZItv.Closed Z0), (ZItv.Closed (Zpos Coq_xH)))
  
  (** val get_itv_cond : ZCond.t -> coq_A -> ZItv.t **)
  
  let get_itv_cond c ab =
    if PolLinZ.FullPolLinZ.require c ab
    then itv_one
    else let c' = ZCond.nnfForRequire (ZCond.Not c) in
         if PolLinZ.FullPolLinZ.require c' ab then itv_zero else itv_unit
  
  (** val get_itv : Expr.t -> coq_A -> ZItv.t **)
  
  let get_itv e ab =
    match Tr.expr e with
    | Some e' -> PolLinZ.FullPolLinZ.get_itv (Obj.magic e') ab
    | None ->
      (match Tr.cond e with
       | Some c -> get_itv_cond c ab
       | None -> ZItv.NotBot (ZItv.Infty, ZItv.Infty))
  
  (** val assume : Expr.t -> bool -> coq_A -> coq_A **)
  
  let assume e b ab =
    match Tr.cond e with
    | Some c ->
      if b
      then let c' = ZCond.nnfForAssume c in PolLinZ.FullPolLinZ.assume c' ab
      else let c' = ZCond.nnfForAssume (ZCond.Not c) in
           PolLinZ.FullPolLinZ.assume c' ab
    | None -> ab
  
  (** val project : coq_A -> PVar.t -> coq_A **)
  
  let project ab x =
    PolLinZ.FullPolLinZ.project ab x
  
  (** val assign : PVar.t -> Expr.t -> coq_A -> coq_A **)
  
  let assign x e ab =
    match Tr.asg x e with
    | Some c -> PolLinZ.FullPolLinZ.guassign x c ab
    | None -> project ab x
 end

