open AffCond
open AffTerm
open BinNums
open CoqAddOn
open Itv
open NumC
open ProgVar

module Cmp : 
 sig 
  type t =
  | Eq
  | Le
  | Lt
  | Neq
  | Ge
  | Gt
  
  val t_rect : 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> t -> 'a1
  
  val t_rec : 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> t -> 'a1
  
  val pr : t -> char list
  
  val dec : t -> ZNum.t -> ZNum.t -> bool
 end

module Expr : 
 sig 
  type binopT =
  | Oadd
  | Osub
  | Omul
  | Odiv
  | Omod
  | Oand
  | Oor
  | Oxor
  | Oshl
  | Oshr
  | Ocmp of Cmp.t
  
  val binopT_rect :
    'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 ->
    (Cmp.t -> 'a1) -> binopT -> 'a1
  
  val binopT_rec :
    'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 -> 'a1 ->
    (Cmp.t -> 'a1) -> binopT -> 'a1
  
  type unopT =
  | Oneg
  | Onot
  
  val unopT_rect : 'a1 -> 'a1 -> unopT -> 'a1
  
  val unopT_rec : 'a1 -> 'a1 -> unopT -> 'a1
  
  type t =
  | Evar of PVar.t
  | Econst of ZNum.t
  | Eitv of ZItv.t
  | Eunknown
  | Eunop of unopT * t
  | Ebinop of binopT * t * t
  
  val t_rect :
    (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (ZItv.t -> 'a1) -> 'a1 -> (unopT ->
    t -> 'a1 -> 'a1) -> (binopT -> t -> 'a1 -> t -> 'a1 -> 'a1) -> t -> 'a1
  
  val t_rec :
    (PVar.t -> 'a1) -> (ZNum.t -> 'a1) -> (ZItv.t -> 'a1) -> 'a1 -> (unopT ->
    t -> 'a1 -> 'a1) -> (binopT -> t -> 'a1 -> t -> 'a1 -> 'a1) -> t -> 'a1
  
  val pr : t -> char list
 end

module Tr : 
 sig 
  val itv2z : Expr.t -> Expr.t
  
  val sub2add : Expr.t -> Expr.t
  
  val simplify : Expr.t -> Expr.t
  
  val bind : 'a1 option -> ('a1 -> 'a2 option) -> 'a2 option
  
  val expr_aux : Expr.t -> ZTerm.t option
  
  val expr : Expr.t -> ZTerm.t option
  
  val cmp : Cmp.t -> ZTerm.t -> ZTerm.t -> ZCond.t
  
  val cond : Expr.t -> ZCond.t option
  
  val asg_itv : PVar.t -> ZItv.t -> ZCond.t option
  
  val asg : PVar.t -> Expr.t -> ZCond.t option
 end

module AbDom : 
 sig 
  type coq_A = PolLinZ.FullPolLinZ.t
  
  val pr : coq_A -> char list
  
  type coq_B = ZNum.t Mem.t
  
  val top : coq_A
  
  val leb : coq_A -> coq_A -> bool
  
  val bottom : coq_A
  
  val isBottom : coq_A -> bool
  
  val join : coq_A -> coq_A -> coq_A
  
  val widen : coq_A -> coq_A -> coq_A
  
  val itv_one : ZItv.t
  
  val itv_zero : ZItv.t
  
  val itv_unit : ZItv.t
  
  val get_itv_cond : ZCond.t -> coq_A -> ZItv.t
  
  val get_itv : Expr.t -> coq_A -> ZItv.t
  
  val assume : Expr.t -> bool -> coq_A -> coq_A
  
  val project : coq_A -> PVar.t -> coq_A
  
  val assign : PVar.t -> Expr.t -> coq_A -> coq_A
 end

