open System.Numerics

type t = BigRational;;

let z = BigRational.Zero;;
let u = BigRational.One;;
let negU = -u

let cmp (x : t) (y : t) = compare x y
let cmpz (q : t) = -q.Sign
let isInt (n : t) = n.Denominator = BigInteger(1)

let abs (x : t) = BigRational.Abs x
let neg (x : t) = -x
let inv (x : t) = u / x

let add (x : t) (y : t) = x + y
let sub (x : t) (y : t) = x - y
let mult (x : t) (y : t) = x * y
let div (x : t) (y : t) = x / y

let mk (den: int) (num: int) = BigRational.FromInt(num) / BigRational.FromInt(den)
let mk1 (num: int) = BigRational.FromInt(num)
let pr (x : t) = x.ToString()

module Z =
struct
	type t = BigInteger

	let z = BigInteger.Zero
	let u = BigInteger.One

	let cmp (x : t) (y : t) = BigInteger.Compare(x, y)
	let equal (x : t) (y : t) = x=y
	let neg (x : t) = -x
	let gcd (x : t) (y : t) = BigInteger.GreatestCommonDivisor(x,y)
	let lAnd (x : t) (y : t) = BigInteger.op_BitwiseAnd(x, y)
	let orL (x : t) (y : t) = BigInteger.op_BitwiseOr(x, y)
	let shiftR (x : t) (y : int) = BigInteger.op_RightShift(x, y)
	let shiftL (x : t) (y : int) = BigInteger.op_LeftShift(x, y)

	let mk (x : int) = BigInteger(x)
	let pr (x : t) = x.ToString()
end

let toZ (x : t) = (x.Numerator, x.Denominator)
let ofZ (num : Z.t) (den : Z.t) = (BigRational.FromBigInt num) / (BigRational.FromBigInt den)

let num (x : t) = x.Numerator
let den (x : t) = x.Denominator

let gcd (nGcd, dGcd) a =
	((if Z.equal Z.u nGcd then
		Z.u
	else
		Z.gcd nGcd (num a)),
	(if Z.equal Z.u dGcd then
		Z.u
	else
		Z.gcd dGcd (den a)))
