(** This module provides means to declare new variables to be used by the library. *)

(** A value of this type keeps track of already allocated variables. *)
type t = {
	b: string Rtree.t;
	nxt: Var.t;
}

val get_b : t -> string Rtree.t
val get_nxt : t -> Var.t

(** Empty value in which no variables have been allocated yet. *)
val nil: t

(** [add b s] adds a variable named [s] to an existing basis [b].
This returns a unique identifier for [s] and a new basis recording the allocation.
Note that [add] does not check whether there already is a variable named [s] in [b]. *)
val add: t -> string -> Var.t * t

(** Create a new basis and add a list of variables to it.
The identifiers are returned in the same order as the strings. *)
val mk: string list -> Var.t list * t

(** Return the name of the variable associated with a variable.
[pr] raises [Invalid_argument "Basis.pr]" if a bad variable is given. *)
val pr: t -> Var.t -> string
