(** This module provides the necessary elements for the Coq development
to call the VPL to build proofs of its results. *)

val coqPosToZ: BinNums.positive -> Num.Z.t
val zToCoqPos: Num.Z.t -> BinNums.positive

val coqZToZ: BinNums.coq_Z -> Num.Z.t
val zToCoqZ: Num.Z.t -> BinNums.coq_Z

val nToNum: NumC.QNum.t -> Num.t
val nToNumC: Num.t -> NumC.QNum.t

val progVarToVar: ProgVar.PVar.t -> Var.t
val varToProgVar: Var.t -> ProgVar.PVar.t

val ltToVec: LinTerm.Lin.t -> Vec.t
val vecToLt: Vec.t -> LinTerm.Lin.t

val cToCstr: CstrC.Cstr.t -> Cstr.t
val cToCstrC: Cstr.t -> CstrC.Cstr.t
val cToQCond: Cstr.t -> AffCond.QCond.t

val cToConsC: Cons.t -> ConsSet.Cs.cT

val pToConsSet: Pol.t -> ConsSet.Cs.t

val cstrPr: CertC.Cert.cstrT -> string
val cToCertC: Cert.t -> CertC.Cert.t
val certCPr: CertC.Cert.t -> string
val joinEltPr: CertC.Cert.joinEltT -> string
val joinPr: CertC.Cert.joinT -> string

val iToItv: Itv.QItv.t -> Pol.itvT

(** Check whether a [ConsSet.Cs.t] and a [Pol.t] describe exactly the
same polyhedra, in such a way that further communication between the
code extracted from Coq and the Ocaml code has a chance to work. *)
val chkSync: ConsSet.Cs.t -> Pol.t -> bool

val isIncl: Pol.t -> Pol.t -> Shadow.inclT
val add: Pol.t -> CstrC.Cstr.t -> Pol.t option * CertC.Cert.t
val addp: Pol.t -> CstrC.Cstr.t -> Shadow.addPrecT
val meet: Pol.t -> Pol.t -> Pol.t option * CertC.Cert.meetT
val join: Pol.t -> Pol.t -> Pol.t * CertC.Cert.joinT
val widen: Pol.t -> Pol.t -> Pol.t * ConsSet.Cs.t
val getItv: Pol.t -> LinTerm.Lin.t -> Shadow.itvT
(* not used any more !
val assign: Pol.t -> Shadow.assignT -> Pol.t * CertC.Cert.asgT
*)
val project: Pol.t -> ProgVar.PVar.t -> Pol.t * CertC.Cert.t
val rename : ProgVar.PVar.t -> ProgVar.PVar.t -> Pol.t -> Pol.t
