module AffTermM = Map.Make (String)

type affTermT = Num.t AffTermM.t

let constant = ""

let create: string -> Num.t -> affTermT = fun x n ->
	if Num.cmpz n = 0 then
		AffTermM.empty
	else
		AffTermM.singleton x n

let negate: affTermT -> affTermT = fun a ->
	let fn n = Num.neg n in
	AffTermM.map fn a

let merge: affTermT -> affTermT -> affTermT = fun a1 a2 ->
	let fn _ opt1 opt2 =
		match opt1, opt2 with
		| None, None -> None
		| Some n, None | None, Some n -> Some n
		| Some n1, Some n2 ->
			let n = Num.add n1 n2 in
			if Num.cmpz n = 0 then
				None
			else
				Some n
	in
	AffTermM.merge fn a1 a2

let buildCons: Env.t -> affTermT -> CmpExt.t -> affTermT -> Env.t * Cstr.t = fun e a1 op a2 ->
	let (a, op1) =
		match op with
		| CmpExt.LE -> (merge a1 (negate a2), Cmp.Le)
		| CmpExt.LT -> (merge a1 (negate a2), Cmp.Lt)
		| CmpExt.EQ -> (merge a1 (negate a2), Cmp.Eq)
		| CmpExt.GE -> (merge a2 (negate a1), Cmp.Le)
		| CmpExt.GT -> (merge a2 (negate a1), Cmp.Lt)
	in
	let cst =
		if AffTermM.mem constant a then
			Num.neg (AffTermM.find constant a)
		else
			Num.z
	in
	let (e1, coefs) =
		let fn str n (e2, a) =
			let (e3, x) = Env.get e2 str in
			(e3, (n, x)::a)
		in
		AffTermM.fold fn (AffTermM.remove constant a) (e, [])
	in
	(e1, Cstr.mk op1 coefs cst)

let pr: affTermT -> string = fun a ->
	let fn x n s =
		let prefix =
			if String.length s = 0 then
				s
			else
				s ^ " + "
		in
		if String.length x = 0 then
			prefix ^ (Num.pr n)
		else
			prefix ^ (Printf.sprintf "%s*%s" (Num.pr n) x)
	in
	let s = AffTermM.fold fn a "" in
	if String.length s = 0 then
		"0"
	else
		s