#!/bin/sh
a=vpl.tgz # name of the generated archive
r=. # where to find the files
d=ocaml # destination directory
jobs=5 # number of jobs in parallel make run
make=make # the name of the make(1) executable

install_lib () {
	cp -R ${r}/lib ${d}/lib

	for f in Glue Support
	do
		rm ${d}/lib/${f}.ml
		rm ${d}/lib/${f}.mli
	done
}

install_test () {
	cp -R ${r}/test ${d}/test

	for f in Glue_t Support_t
	do
		rm ${d}/test/${f}.ml
		rm ${d}/test/${f}.mli
	done

	rm ${d}/test/${f}.ml
	cat > ${d}/test/Coq_t.ml << EOF
let ts: T.testT
= T.suite "coq" []
EOF

}

install_helpers () {
	cp -R ${r}/msort ${d}/msort
	cp ${r}/ocamldep ${d}/ocamldep
}

install_buildsys () {
	mkdir ${d}/obj
	mkdir ${d}/obj/msort
	cp ${r}/obj/msort/Makefile ${d}/obj/msort/Makefile
	sed -e '/#$/d' -e 's/ -I \.\.\/extract//g' -e 's/ \.\.\/extract//' ${r}/obj/Makefile> ${d}/obj/Makefile
	sed '/#$/d' ${r}/Makefile > ${d}/Makefile
}

extract () {
	echo "> extracting to $d"

	if test -d $d
	then
		rm -r -f $d
	fi

	mkdir -p $d
	install_lib
	install_test
	install_helpers
	install_buildsys
}

compile () {
	echo "> compiling"

	cd $d
	$make -j $jobs vpl
	cd ..
}

run () {
	echo "> running the test suite"

	cd $d
	$make -j $jobs check
	cd ..
}

clean () {
	echo "> cleaning"

	cd $d
	$make -j $jobs clean > /dev/null
	cd ..
}

pack () {
	echo "> building the archive $a"

	if test -f $a
	then
		rm $a
	fi

	tar -czf $a $d
}

extract
compile
run
clean
pack

exit 0

