type sign_t = P | Z | N

let sign a = if a < 0 then N else if a > 0 then P else Z
let pr = function P -> "P" | Z -> "Z" | N -> "N"

(* Num.cmp *)
let cmpTs: T.testT
=
	let chk (t, n1, n2, s) = fun state ->
		let r = sign (Num.cmp n1 n2) in
		if r = s then
			T.succeed state
		else
			let e = "Num.cmp " ^ (Num.pr n1) ^ " " ^ (Num.pr n2) ^ ": " ^ (pr r) in
			T.fail t e state
	in
	let tcs = [
		"zz0", Num.z, Num.z, Z;
		"zu0", Num.z, Num.u, N;
		"zu1", Num.z, Num.negU, P;
		"uz0", Num.u, Num.z, P;
		"uz1", Num.negU, Num.z, N;
		"uu0", Num.u, Num.u, Z;
		"uu1", Num.u, Num.negU, P;
		"uu2", Num.negU, Num.u, N;
		"pz0", Num.mk1 2, Num.z, P;
		"zp0", Num.z, Num.mk1 2, N;
		"pp0", Num.mk1 2, Num.mk1 2, Z;
		"pp1", Num.mk1 2, Num.mk1 3, N;
		"pp2", Num.mk1 3, Num.mk1 2, P;
		"nn0", Num.mk1 (-2), Num.mk1 (-2), Z;
		"nn1", Num.mk1 (-2), Num.mk1 (-3), P;
		"nn2", Num.mk1 (-3), Num.mk1 (-2), N;
		"pn0", Num.mk1 2, Num.mk1 (-2), P;
		"np0", Num.mk1 (-2), Num.mk1 2, N
	] in
	T.suite "cmp" (List.map chk tcs)

(* Num.cmpz *)
let cmpzTs: T.testT
=
	let chk (t, n, s) = fun state ->
		let r = sign (Num.cmpz n) in
		if r = s then
			T.succeed state
		else
			let e = "Num.cmpz " ^ (Num.pr n) ^ ": " ^ (pr r) in
			T.fail t e state
	in
	let tcs = [
		"z", Num.z, Z;
		"u0", Num.u, N;
		"u1", Num.negU, P;
		"p0", Num.mk1 3, N;
		"p1", Num.mk 10 2, N;
		"n0", Num.mk1 (-4), P
	] in
	T.suite "cmpz" (List.map chk tcs)

(* Num.neg *)
let negTs: T.testT
=
	let chk_res (t, n1, n2) = fun state ->
		let r = Num.neg n1 in
		if Num.cmp n2 r = 0 then
			T.succeed state
		else
			let e = "Num.neg " ^ (Num.pr n1) ^ " = " ^ (Num.pr r) in
			T.fail t e state
	in
	let chk_comp (t, n1, _) = fun state ->
		let r = Num.neg (Num.neg n1) in
		if Num.cmp n1 r = 0 then
			T.succeed state
		else
			let e = "Num.neg (Num.neg " ^ (Num.pr n1) ^ ") = " ^ (Num.pr r) in
			T.fail t e state
	in
	let tcs = [
		"z", Num.z, Num.z;
		"u", Num.u, Num.negU;
		"p0", Num.mk1 2, Num.mk1 (-2);
		"p1", Num.mk1 5, Num.mk1 (-5);
		"n0", Num.negU, Num.u;
		"n1", Num.mk1 (-15), Num.mk1 15
	] in
	T.suite "neg" [
		T.suite "res" (List.map chk_res tcs);
		T.suite "comp" (List.map chk_comp tcs)
	]

(* Num.inv *)
(*
let invKoTs: T.testT
= fun state ->
	let r = Num.inv Num.z in
	if not (Num.isReal r) then
		T.succeed state
	else
		let e = "Num.inv Num.z = " ^ (Num.pr r) in
		T.fail "ko" e state
*)

let invOkTs: T.testT
=
	let chk_res (t, n1, n2) = fun state ->
		let n = Num.inv n1 in
		if Num.cmp n2 n = 0 then
			T.succeed state
		else
			let e = "Num.inv " ^ (Num.pr n1) ^ " = " ^ (Num.pr n) in
			T.fail t e state
	in
	let chk_comp (t, n1, _) = fun state ->
		let n = Num.inv (Num.inv n1) in
		if Num.cmp n1 n = 0 then
			T.succeed state
		else
			let e = "Num.inv (Num.inv " ^ (Num.pr n1) ^ ") = " ^ (Num.pr n) in
			T.fail t e state
	in
	let tcs = [
		"u0", Num.u, Num.u;
		"u1", Num.negU, Num.negU;
		"p0", Num.mk1 2, Num.mk 2 1;
		"n0", Num.mk1 (-2), Num.mk 2 (-1)
	] in
	T.suite "ok" [
		T.suite "res" (List.map chk_res tcs);
		T.suite "comp" (List.map chk_comp tcs)
	]

(* DM note: invKoTs does a divide by zero in F# *)
let invTs: T.testT
= T.suite "inv" [(* invKoTs; *) invOkTs]

(* Num.add *)
let addTs: T.testT
=
	let chk_res (t, n1, n2, n) = fun state ->
		let r = Num.add n1 n2 in
		if Num.cmp n r = 0 then
			T.succeed state
		else
			let e = (Num.pr n1) ^ " + " ^ (Num.pr n2) ^ " = " ^ (Num.pr r) in
			T.fail t e state
	in
	let chk_sym (t, n1, n2, _) = fun state ->
		let r1 = Num.add n1 n2 in
		let r2 = Num.add n2 n1 in
		if Num.cmp r1 r2 = 0 then
			T.succeed state
		else
			let e1 = (Num.pr n1) ^ " + " ^ (Num.pr n2) ^ " = " ^ (Num.pr r1) in
			let e2 = (Num.pr n2) ^ " + " ^ (Num.pr n1) ^ " = " ^ (Num.pr r2) in
			T.fail t (e1 ^ ", " ^ e2) state
	in
	let tcs = [
		"zz", Num.z, Num.z, Num.z;
		"u0", Num.z, Num.u, Num.u;
		"u1", Num.u, Num.u, Num.mk1 2;
		"p0", Num.mk1 2, Num.u, Num.mk1 3;
		"p1", Num.mk1 10, Num.mk1 15, Num.mk1 25;
		"n0", Num.negU, Num.z, Num.negU;
		"n1", Num.negU, Num.u, Num.z;
		"n2", Num.mk1 (-5), Num.u, Num.mk1 (-4)
	] in
	T.suite "add" [
		T.suite "res" (List.map chk_res tcs);
		T.suite "sym" (List.map chk_sym tcs)
	]

(* Num.mult *)
let multTs: T.testT
=
	let chk_res (t, n1, n2, n) = fun state ->
		let r = Num.mult n1 n2 in
		if Num.cmp n r = 0 then
			T.succeed state
		else	
			let e = (Num.pr n1) ^ " + " ^ (Num.pr n2) ^ " = " ^ (Num.pr r) in
			T.fail t e state
	in
	let chk_sym (t, n1, n2, _) = fun state ->
		let r1 = Num.mult n1 n2 in
		let r2 = Num.mult n2 n1 in
		if Num.cmp r1 r2 = 0 then
			T.succeed state
		else	
			let e1 = (Num.pr n1) ^ " + " ^ (Num.pr n2) ^ " = " ^ (Num.pr r1) in
			let e2 = (Num.pr n2) ^ " + " ^ (Num.pr n1) ^ " = " ^ (Num.pr r2) in
			T.fail t (e1 ^ ", " ^ e2) state
	in
	let tcs = [
		"z0", Num.z, Num.z, Num.z;
		"z1", Num.z, Num.u, Num.z;
		"z2", Num.z, Num.mk1 2, Num.z;
		"z3", Num.z, Num.mk1 (-2), Num.z;
		"u0", Num.u, Num.u, Num.u;
		"u1", Num.u, Num.mk1 2, Num.mk1 2;
		"u2", Num.u, Num.mk1 (-2), Num.mk1 (-2);
		"p0", Num.mk1 2, Num.mk1 3, Num.mk1 6;
		"p1", Num.mk1 10, Num.mk1 15, Num.mk1 150;
		"n0", Num.negU, Num.u, Num.negU;
		"n1", Num.negU, Num.negU, Num.u;
		"n2", Num.mk1 (-5), Num.mk1 5, Num.mk1 (-25);
		"n2", Num.mk1 (-5), Num.mk1 (-5), Num.mk1 25
	] in
	T.suite "mult" [
		T.suite "res" (List.map chk_res tcs);
		T.suite "sym" (List.map chk_sym tcs)
	]

(* Num.div *)
(*
let divKoTs: T.testT
=
	let chk (t, n, d) = fun state ->
		let r = Num.div n d in
		if not (Num.isReal r) then
			T.succeed state
		else			
			let e = (Num.pr r) ^ "is real" in
			T.fail t e state
	in
	let tcs = [
		"zz", Num.z, Num.z;
		"u0", Num.u, Num.z;
		"u1", Num.negU, Num.z;
		"p0", Num.mk1 2, Num.z;
		"n0", Num.mk1 (-5), Num.z
	] in
	T.suite "ko" (List.map chk tcs)
*)

let divOkTs: T.testT
=
	let chk (t, n, d, r) = fun state ->
		let r1 = Num.div n d in
		if Num.cmp r r1 = 0 then
			T.succeed state
		else	
			let e = "Num.div " ^ (Num.pr n) ^ " " ^ (Num.pr d) ^ " = " ^ (Num.pr r1) in
			T.fail t e state
	in
	let tcs = [
		"u0", Num.z, Num.u, Num.z;
		"u1", Num.z, Num.negU, Num.z;
		"u2", Num.u, Num.u, Num.u;
		"u3", Num.negU, Num.u, Num.negU;
		"u4", Num.mk1 2, Num.u, Num.mk1 2;
		"u5", Num.mk1 2, Num.negU, Num.mk1 (-2);
		"p0", Num.mk1 4, Num.mk1 2, Num.mk1 2;
		"p1", Num.mk1 3, Num.mk1 2, Num.mk 2 3;
		"p2", Num.mk1 6, Num.mk1 4, Num.mk 2 3;
		"n0", Num.mk1 (-4), Num.mk1 2, Num.mk1 (-2);
		"n1", Num.mk1 4, Num.mk1 (-2), Num.mk1 (-2);
		"n2", Num.mk1 (-4), Num.mk1 (-2), Num.mk1 2
	] in
	T.suite "ok" (List.map chk tcs)

(* DM NOTE: divide by zero crashes F# *)
let divTs: T.testT
= T.suite "div" [(*divKoTs; *) divOkTs]

let ts: T.testT
= T.suite "Num" [cmpTs; cmpzTs; negTs; invTs; addTs; multTs; divTs]
