open Itv
open AffCond.ZCond
open PolZ
open PolZ.Expr

let x = Var.fromInt 1
let y = Var.fromInt 2

let varPr: Var.t -> string
= fun _x -> List.assoc _x [x, "x"; y, "y"]

let v2p: Var.t -> BinNums.positive
= fun x -> Support.zToCoqPos (Num.Z.mk (Var.toInt x))

let c2i: int -> ZItv.t
= fun i ->
	let z = Support.zToCoqZ (Num.Z.mk i) in
	ZItv.NotBot (ZItv.Closed z, ZItv.Closed z)

let itv: int -> int -> ZItv.t
= fun l u ->
	let lz = Support.zToCoqZ (Num.Z.mk l) in
	let uz = Support.zToCoqZ (Num.Z.mk u) in
	ZItv.NotBot (ZItv.Closed lz, ZItv.Closed uz)

let mkc t v c =
    Cstr.mk t (List.map (function (i, x) -> (Num.mk1 i, x)) v) (Num.mk1 c)

let eq = mkc Cstr.Eq
let le = mkc Cstr.Le
let lt = mkc Cstr.Lt

let p: Cstr.t list -> PolZ.AbDom.coq_A
= fun l ->
	let f: Cstr.t -> Cstr.t
	= fun c -> {c with Cstr.v = Rtree.Sub (c.Cstr.v, Num.z, Rtree.Nil)}
	in
	let l' = List.map f l in
	match Pol.mk l' with
	| None -> Pervasives.failwith "PolZ_t.p"
	| Some p -> {
		PolLinZ.FullPolLinZ.pol = Glue.pToPolC p;
		PolLinZ.FullPolLinZ.renaming = MSetPositive.PositiveSet.empty
	}

let bot: PolZ.AbDom.coq_A
= {
	PolLinZ.FullPolLinZ.pol = None;
	PolLinZ.FullPolLinZ.renaming = MSetPositive.PositiveSet.empty
}

(* PolZ.AbDom.assume *)
let assumeTs: T.testT
=	let chk (nm, b, ex, p, p0): T.stateT -> T.stateT
	= fun state ->
		let p' = PolZ.AbDom.assume ex b p in
		if p' = p0
		then T.succeed state
		else T.fail nm (CoqPr.charListTr (*(PolC.pr p')*)[]) state
	in
	let tcs: (string * bool * Expr.t * PolZ.AbDom.coq_A * PolZ.AbDom.coq_A) list
	= [
	"simpl0", false,
		Ebinop (Ocmp Cmp.Le, Evar (v2p x), Eitv (c2i 3)),
		p [eq [1, x] 2; eq [1, y] 1],
		bot;

	"compl0", false,
		Ebinop (Ocmp Cmp.Gt, Evar (v2p x), Eitv (c2i 0)),
		p [eq [1, x] 2; eq [1, y] 1],
		bot
	] in
	T.suite "assume" (List.map chk tcs)

(* PolZ.AbDom.get_itv *)
let get_itvTs: T.testT
=	let chk (nm, ex, p, i): T.stateT -> T.stateT
	= fun state ->
		let i' = PolZ.AbDom.get_itv ex p in
		if i = i'
		then T.succeed state
		else
			let e = "failure" in
			T.fail nm e state
	in
	let tcs: (string * Expr.t * PolZ.AbDom.coq_A * ZItv.t) list
	= [
	"triv0", Eitv (c2i 0),
		p [],
		c2i 0;

	"triv1", Eitv (c2i 1),
		p [],
		c2i 1;

	"var0", Evar (v2p x),
		p [eq [1, x] 2],
		c2i 2;

	"cstr0", Ebinop (Ocmp Cmp.Le, Evar (v2p x), Eitv (c2i 1)),
		p [],
		itv 0 1;

	"cstr1", Ebinop (Ocmp Cmp.Le, Evar (v2p x), Eitv (c2i 1)),
		p [eq [1, x] 0],
		itv 1 1;

	"cstr2", Ebinop (Ocmp Cmp.Le, Evar (v2p x), Eitv (c2i 1)),
		p [eq [1, x] 2],
		itv 0 0
	] in
	T.suite "get_itv" (List.map chk tcs)

(* PolZ.AbDom.project *)
let projectTs : T.testT
=	let polZEq : PolZ.AbDom.coq_A -> PolZ.AbDom.coq_A -> bool
	= fun p p' -> PolZ.AbDom.leb p p' && PolZ.AbDom.leb p' p
	in
	let chk (nm, x, p, ep) : T.stateT -> T.stateT
	= fun st ->
		let ap = PolZ.AbDom.project p x in
		if polZEq ep ap
		then T.succeed st
		else
			let e = "failure" in
			T.fail nm e st
	in
	let tcs : (string * ProgVar.PVar.t * PolZ.AbDom.coq_A * PolZ.AbDom.coq_A) list
	= [
		"id0", Support.varToProgVar x, p [], p [];
		"eq0", Support.varToProgVar x, p [eq [1, x] 0], p [];
		"eq1", Support.varToProgVar x, p [eq [1, y] 0], p [eq [1, y] 0];
	] in
	T.suite "project" (List.map chk tcs)

let ts: T.testT
= T.suite "PolZ" [assumeTs; get_itvTs; projectTs]
