let x : Var.t = Var.fromInt 1
let y : Var.t = Var.fromInt 2
let z : Var.t = Var.fromInt 3

let cstrMk : Cstr.cmpT -> (int * Var.t) list -> int -> Cstr.t
=	let tr : int * Var.t -> Num.t * Var.t
	= fun (i, x) -> (Num.mk1 i, x)
	in
	fun c l n -> Cstr.mk c (List.map tr l) (Num.mk1 n)

let le = cstrMk Cstr.Le
let eq = cstrMk Cstr.Eq

let v : int array -> Num.t array
= fun a -> Array.init (Array.length a) (fun i -> Num.mk1 a.(i))

let projConsTs : T.testT
=	let chk : string * (int * Cstr.t) list * Var.t * Psplx.vecT
		-> T.stateT -> T.stateT
	= fun (nm, l, x, ec) st ->
		let ac = Psplx.projCons l x in
		if Psplx.vecEq ac ec
		then T.succeed st
		else
			let e = Printf.sprintf "expected: %s\ngot: %s\n"
				(Psplx.vecPr ec) (Psplx.vecPr ac)
			in
			T.fail nm e st
	in
	let tcs : (string * (int * Cstr.t) list * Var.t * Psplx.vecT) list
	=	let c1 = le [1, x] 0 in
		let c2 = le [2, x] 1 in
		let c3 = le [1, x; 1, y] 1 in
		let c4 = le [1, x; -1, y] 1 in
	[
		"trivial0", [1, c1], x, v [|1; 0|];

		"trivial1", [1, c2], x, v [|2; 0|];

		"trivial2", [1, c1], y, v [|0; 0|];

		"comb0", [1, c3; 2, c4], y, v [|1; -1; 0|];

	] in
	T.suite "projCons" (List.map chk tcs)

let normConsTs : T.testT
=	let chk : string * (int * Cstr.t) list * Cstr.t * Psplx.vecT
		-> T.stateT -> T.stateT
	= fun (nm, l, c, ec) st ->
		let ac = Psplx.normCons l c in
		if Psplx.vecEq ac ec
		then T.succeed st
		else
			let e = Printf.sprintf "expected: %s\ngot: %s\n"
				(Psplx.vecPr ec) (Psplx.vecPr ac)
			in
			T.fail nm e st
	in
	let tcs : (string * (int * Cstr.t) list * Cstr.t * Psplx.vecT) list
	=	let c1 = le [1, x; 1, y; 1, z] 2 in
		let c2 = le [1, x; 1, y; -1, z] 2 in
		let c3 = le [-3, x; -1, y; 1, z] (-3) in
		let c4 = le [-3, x; -1, y; -1, z] (-3) in
		let c5 = le [-1, x; 1, y; 2, z] 2 in
		let c6 = le [-1, x; 1, y; -2, z] 2 in
	[
		"king12", [1, c1; 2, c2; 3, c3; 4, c4; 5, c5; 6, c6; 7, le [] 1],
			eq [-3, x; 1, y] 2,
			v [|2; 2; 2; 2; 8; 8; 2; 1|]
	] in
	T.suite "normCons" (List.map chk tcs)

let tabMkTs : T.testT
=	let chk : string * (int * Cstr.t) list * Var.t list * Cstr.t * Psplx.tabT
		-> T.stateT -> T.stateT
	= fun (nm, l, xl, c, et) st ->
		let at = Psplx.tabMk l xl c in
		if Psplx.tabEq et at
		then T.succeed st
		else
			let e = Psplx.tabPr at in
			T.fail nm e st
	in
	let tcs : (string * (int * Cstr.t) list * Var.t list * Cstr.t
		* Psplx.tabT) list
	=	let c1 = le [1, x; 1, y; 1, z] 2 in
		let c2 = le [1, x; 1, y; -1, z] 2 in
		let c3 = le [-3, x; -1, y; 1, z] (-3) in
		let c4 = le [-3, x; -1, y; -1, z] (-3) in
		let c5 = le [-1, x; 1, y; 2, z] 2 in
		let c6 = le [-1, x; 1, y; -2, z] 2 in
	[
		"king12", [1, c1; 2, c2; 3, c3; 4, c4; 5, c5; 6, c6; 7, le [] 1],
			[z], eq [-3, x; 1, y] 2, {
				Psplx.obj = [|
					v [|0;-1;-1|];
					v [|0;-1;-1|];
					v [|0;3;1|];
					v [|0;3;1|];
					v [|0;1;-1|];
					v [|0;1;-1|];
					v [|0;0;0|];
					v [|0;0;0|]
				|];
				Psplx.pval = v [|0;1;1|];
				Psplx.cs = [|
					{Psplx.xb = -1; Psplx.v = v [|2;2;2;2;8;8;2;1|]};
					{Psplx.xb = -1; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
				|]
			};
	] in
	T.suite "tabMk" (List.map chk tcs)

let (>:) : 'a -> ('a -> 'b) -> 'b
= fun x f -> f x

let (-:) : ('a -> 'b) -> ('b -> 'c) -> 'a -> 'c
= fun f g x -> g (f x)

let ifFailed : string -> string -> bool -> T.stateT -> T.stateT
= fun nm e r st ->
	if r then T.succeed st else T.fail nm e st

let pivotTs : T.testT
=	let chk : string * int * Psplx.varT * Psplx.tabT * Psplx.tabT
		-> T.stateT -> T.stateT
	= fun (nm, xi, xj, t, et) ->
		Psplx.pivot xi xj t >: fun res ->
		Psplx.tabEq et res >:
		let e = Printf.sprintf "expected:\n%s\ngot:\n%s\n"
			(Psplx.tabPr et) (Psplx.tabPr res)
		in ifFailed nm e
	in
	let tcs : (string * int * Psplx.varT * Psplx.tabT * Psplx.tabT) list
	=
	[
		"king12_trivial", 0, 6, {
				Psplx.obj = [|
					v [|0;-1;-1|];
					v [|0;-1;-1|];
					v [|0;3;1|];
					v [|0;3;1|];
					v [|0;1;-1|];
					v [|0;1;-1|];
					v [|0;0;0|];
					v [|0;0;0|]
				|];
				Psplx.pval = v [|0;1;1|];
				Psplx.cs = [|
					{Psplx.xb = -1; Psplx.v = v [|2;2;2;2;8;8;2;1|]};
					{Psplx.xb = -1; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
				|]
			},{
				Psplx.obj = [|
					v [|0;-1;-1|];
					v [|0;-1;-1|];
					v [|0;3;1|];
					v [|0;3;1|];
					v [|0;1;-1|];
					v [|0;1;-1|];
					v [|0;0;0|];
					v [|0;0;0|]
				|];
				Psplx.pval = v [|0;1;1|];
				Psplx.cs = [|
					{Psplx.xb = 6; Psplx.v = [|Num.u; Num.u; Num.u; Num.u;
						Num.mk1 4; Num.mk1 4; Num.u; Num.mk 2 1|]};
					{Psplx.xb = -1; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
				|]
			}
	] in
	T.suite "pivot" (List.map chk tcs)

let tabInitTs : T.testT
=	let chk : string * Psplx.tabT * Psplx.tabT -> T.stateT -> T.stateT
	= fun (nm, tab, etab) st ->
		let atab = Psplx.tabInit tab in
		if Psplx.tabEq etab atab
		then T.succeed st
		else
			let e = Printf.sprintf "expected:\n%s\ngot:\n%s\n"
				(Psplx.tabPr etab) (Psplx.tabPr atab)
			in
			T.fail nm e st
	in
	let tcs : (string * Psplx.tabT * Psplx.tabT) list
	= [
		"king12", {
			Psplx.obj = [|
				v [|0;-1;-1|];
				v [|0;-1;-1|];
				v [|0;3;1|];
				v [|0;3;1|];
				v [|0;1;-1|];
				v [|0;1;-1|];
				v [|0;0;0|];
				v [|0;0;0|]
			|];
			Psplx.pval = v [|0;1;1|];
			Psplx.cs = [|
				{Psplx.xb = -1; Psplx.v = v [|2;2;2;2;8;8;2;1|]};
				{Psplx.xb = -1; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
			|]
		}, {
			Psplx.obj = [|
				v [|0;0;0|];
				v [|0;-2;-2|];
				v [|0;4;2|];
				v [|0;2;0|];
				v [|0;3;1|];
				v [|0;-1;-3|];
				v [|0;0;0|];
				v [|0;0;0|]
			|];
			Psplx.pval = [|Num.z; Num.u; Num.u|];
			Psplx.cs = [|
				{Psplx.xb = 6; Psplx.v = [|Num.z; Num.mk1 2; Num.z; Num.mk1 2;
					Num.mk1 2; Num.mk1 6; Num.u; Num.mk 2 1|]};
				{Psplx.xb = 0; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
			|]
		}
	] in
	T.suite "tabInit" (List.map chk tcs)

let minTs : T.testT
=	let chkDep : string * Psplx.tabT * Psplx.varT list -> T.stateT -> T.stateT
	=	let listEq : int list -> int list -> bool
		=	let sort = List.sort Pervasives.compare in
			fun l l' ->
				if List.length l <> List.length l'
				then false
				else List.for_all2 (=) (sort l) (sort l')
		in
		fun (nm, tab, xl) st ->
			let atab = Psplx.min Psplx.Bland.strgy (Psplx.tabInit tab) in
			if listEq xl (Psplx.tabDepVars atab)
			then T.succeed st
			else T.fail nm (Psplx.tabPr atab) st
	in
	let tcs : (string * Psplx.tabT * Psplx.varT list) list
	= [
		"king12", {
			Psplx.obj = [|
				v [|0;-1;-1|];
				v [|0;-1;-1|];
				v [|0;3;1|];
				v [|0;3;1|];
				v [|0;1;-1|];
				v [|0;1;-1|];
				v [|0;0;0|];
				v [|0;0;0|]
			|];
			Psplx.pval = v [|0;1;1|];
			Psplx.cs = [|
				{Psplx.xb = -1; Psplx.v = v [|2;2;2;2;8;8;2;1|]};
				{Psplx.xb = -1; Psplx.v = v [|1;-1;1;-1;2;-2;0;0|]};
			|]
		}, [0; 1]
	] in
	T.suite "min" (List.map chkDep tcs)

let nextDeltaTs : T.testT
=	let chk : string * Psplx.pObjT * Psplx.pValT * int * Num.t
		-> T.stateT -> T.stateT
	= fun (nm, po, pv, i, n) st ->
		let n' = Psplx.nextDelta po pv i in
		if Num.cmp n n' = 0
		then T.succeed st
		else
			let e = Printf.sprintf "got %s instead of %s" (Num.pr n') (Num.pr n) in
			T.fail nm e st
	in
	let tcs : (string * Psplx.pObjT * Psplx.pValT * int * Num.t) list
	= [
	"king12", [|
		v [|0; 0; 0|];
		v [|0; 0; 0|];
		v [|0; 4; 2|];
		v [|0; 4; 2|];
		v [|0; 5; 3|];
		v [|0; 5; 3|];
		v [|0; 1; 1|];
		v [|0; 2; 2|]
	|], v [|0; 1; 1|], 1, Num.mk 2 ~-1
	] in
	T.suite "nextDelta" (List.map chk tcs)

let ts : T.testT
= T.suite "Psplx" [
	projConsTs;
	normConsTs;
	tabMkTs;
	pivotTs;
	tabInitTs;
	minTs;
	nextDeltaTs;
]
