open QArith_base

let coqPosTcs: (string * Num.Z.t * BinNums.positive) list
= [
	"u", Num.Z.u, BinNums.Coq_xH;
	"two", Num.Z.mk 2, BinNums.Coq_xO BinNums.Coq_xH;
	"three", Num.Z.mk 3, BinNums.Coq_xI BinNums.Coq_xH;
	"four", Num.Z.mk 4, BinNums.Coq_xO (BinNums.Coq_xO BinNums.Coq_xH);
	"five", Num.Z.mk 5, BinNums.Coq_xI (BinNums.Coq_xO BinNums.Coq_xH);
	"six", Num.Z.mk 6, BinNums.Coq_xO (BinNums.Coq_xI BinNums.Coq_xH);
	"seven", Num.Z.mk 7, BinNums.Coq_xI (BinNums.Coq_xI BinNums.Coq_xH);
	"nine", Num.Z.mk 9, BinNums.Coq_xI (BinNums.Coq_xO (BinNums.Coq_xO BinNums.Coq_xH))
]

(* Support.coqPosToZ *)
let coqPosToZTs: T.testT
=
	let chk (name, z, p) = fun state ->
		let zCalc = Support.coqPosToZ p in
		if Num.Z.cmp z zCalc = 0 then
			T.succeed state
		else
			let err =
				Printf.sprintf "expected: %s, but got: %s for %s"
					(Num.Z.pr z) (Num.Z.pr zCalc) (CoqPr.posPr p)
			in
			T.fail name err state
	in
	T.suite "coqPosToZ" (List.map chk coqPosTcs)

(* Support.zToCoqPos *)
let zToCoqPosOkTs: T.testT
=
	let chk (t, p, s) = fun state ->
		let r = Support.zToCoqPos p in
		if r = s then
			T.succeed state
		else
			let e = "zToCoqPos " ^ (Num.Z.pr p) ^ " = " ^ (CoqPr.posPr r) in
			T.fail t e state
	in
	T.suite "ok" (List.map chk coqPosTcs)

let zToCoqPosKoTs: T.testT
=
	let chk (t, n) = fun state ->
		try
			let _ = Support.zToCoqPos n in
			T.fail t "no exception raised" state
		with
		| Invalid_argument "Support.zToCoqPos" -> T.succeed state
		| _ -> T.fail t "unexpected exception raised" state
	in
	let tcs = [
		"z", Num.Z.z;
		"negU", Num.Z.mk (-1);
		"minus_two", Num.Z.mk (-2)
	] in
	T.suite "ko" (List.map chk tcs)

let zToCoqPosTs: T.testT
= T.suite "zToCoqPos" [zToCoqPosOkTs; zToCoqPosKoTs]


let coqZTcs: (string * Num.Z.t * BinNums.coq_Z) list
= [
	"z", Num.Z.z, BinNums.Z0;
	"u", Num.Z.u, BinNums.Zpos BinNums.Coq_xH;
	"two", Num.Z.mk 2, BinNums.Zpos (BinNums.Coq_xO BinNums.Coq_xH);
	"three", Num.Z.mk 3, BinNums.Zpos (BinNums.Coq_xI BinNums.Coq_xH);
	"negU", Num.Z.mk (-1), BinNums.Zneg BinNums.Coq_xH;
	"negTwo", Num.Z.mk (-2), BinNums.Zneg (BinNums.Coq_xO BinNums.Coq_xH)
]

(* Support.coqZToZ *)
let coqZToZTs: T.testT
=
	let chk (name, z, coqZ) = fun state ->
		let zCalc = Support.coqZToZ coqZ in
		if Num.Z.cmp z zCalc = 0 then
			T.succeed state
		else
			let err =
				Printf.sprintf "expected: %s, but got: %s for %s"
					(Num.Z.pr z) (Num.Z.pr zCalc) (CoqPr.zPr coqZ) in
			T.fail name err state
	in
	T.suite "coqZToZ" (List.map chk coqZTcs)

(* Support.zToCoqZ *)
let zToCoqZTs: T.testT
=
	let chk (t, z, s) = fun state ->
		let r = Support.zToCoqZ z in
		if r = s then
			T.succeed state
		else
			let e = "zToCoqZ " ^ (Num.Z.pr z) ^ " = " ^ (CoqPr.zPr r) in
			T.fail t e state
	in
	T.suite "zToCoqZ" (List.map chk coqZTcs)

let numCTcs: (string * Num.t * NumC.QNum.t) list
= [
	"z", Num.z, { QArith_base.coq_Qnum = BinNums.Z0;
		QArith_base.coq_Qden = BinNums.Coq_xH };
	"u", Num.u, { QArith_base.coq_Qnum = BinNums.Zpos BinNums.Coq_xH;
		QArith_base.coq_Qden = BinNums.Coq_xH };
	"negU", Num.negU, { QArith_base.coq_Qnum = BinNums.Zneg BinNums.Coq_xH;
		QArith_base.coq_Qden = BinNums.Coq_xH }
]

(* Support.nToNum *)
let nToNumTs: T.testT
=
	let chk (name, n, coqN) = fun state ->
		let nCalc = Support.nToNum coqN in
		if Num.cmp n nCalc = 0 then
			T.succeed state
		else
			let err =
				Printf.sprintf "expected: %s, but got: %s for %s"
					(Num.pr n) (Num.pr nCalc) (CoqPr.charListTr (NumC.QNum.pr coqN))
			in
			T.fail name err state
	in
	T.suite "numCToNum" (List.map chk numCTcs)

(* Support.nToNumC *)
let nToNumCTs: T.testT
=
	let chk (t, n, s) = fun state ->
		let r = Support.nToNumC n in
		if r = s then
			T.succeed state
		else
			let e =
				Printf.sprintf "nToNumC %s = %s"
					(Num.pr n) (CoqPr.charListTr (NumC.QNum.pr r))
			in
			T.fail t e state
	in
	T.suite "nToNum" (List.map chk numCTcs)

let vars = T.pathGen 3
let [xV; yV; zV] = vars
let [xP; yP; zP] = List.map Support.varToProgVar vars
let varPr = fun x -> List.assoc x [xV, "x"; yV, "y"; zV, "z"]

let ltTcs: (string * Vec.t * LinTerm.Lin.t) list
= [
	"nil", Vec.nil, LinTerm.Lin.import [];
	"one1", Vec.mk [Num.u, xV], LinTerm.Lin.import [xP, Support.nToNumC Num.u];
	"one2", Vec.mk [Num.u, yV], LinTerm.Lin.import [yP, Support.nToNumC Num.u];
	"one3", Vec.mk [Num.u, zV], LinTerm.Lin.import [zP, Support.nToNumC Num.u];
	"two1", Vec.mk [Num.u, xV; Num.mk1 2, yV],
		LinTerm.Lin.import [xP, Support.nToNumC Num.u; yP, Support.nToNumC (Num.mk1 2)];
	"two2", Vec.mk [Num.u, yV; Num.mk1 2, zV],
		LinTerm.Lin.import [yP, Support.nToNumC Num.u; zP, Support.nToNumC (Num.mk1 2)];
	"two1", Vec.mk [Num.u, xV; Num.mk1 2, zV],
		LinTerm.Lin.import [xP, Support.nToNumC Num.u; zP, Support.nToNumC (Num.mk1 2)]
]

(* Support.ltToVec *)
let ltToVecTs
=
	let chk (name, vRef, lt) = fun state ->
		let v1 = Support.ltToVec lt in
		if Vec.isEq vRef v1
		then T.succeed state
		else
			let e = Printf.sprintf "expected: %s\nbut got: %s\n"
				(Vec.pr varPr vRef) (Vec.pr varPr v1)
			in
			T.fail name e state
	in
	T.suite "ltToVec" (List.map chk ltTcs)

(* Support.vecToLt *)
let vecToLtTs
=
	let chk (name, v, ltRef) = fun state ->
		let lt = Support.vecToLt v in
		let lt1 = LinTerm.Lin.export lt in
		let ltRef1 = LinTerm.Lin.export ltRef in
		let pairEq (x1, n1) (x2, n2)
		= ProgVar.PVar.isEq x1 x2 && NumC.QNum.eqDec n1 n2
		in
		let isIncl lt1 lt2
		= List.for_all (fun p1 -> List.exists (pairEq p1) lt2) lt1
		in
		if isIncl lt1 ltRef1 && isIncl ltRef1 lt1
		then T.succeed state
		else
			let e = Printf.sprintf "expected: %s\nbut got: %s\n"
				(CoqPr.charListTr (LinTerm.Lin.pr ltRef))
				(CoqPr.charListTr (LinTerm.Lin.pr lt))
			in
			T.fail name e state
	in
	T.suite "vecToLtTs" (List.map chk ltTcs)

let ts: T.testT
= T.suite "Support" [
	coqPosToZTs; zToCoqPosTs;
	coqZToZTs; zToCoqZTs;
	nToNumTs; nToNumCTs;
	ltToVecTs; vecToLtTs
]
