type sign_t = P | Z | N

let sign a = if a < 0 then N else if a > 0 then P else Z
let pr = function P -> "P" | Z -> "Z" | N -> "N"

(* Num.Z.cmp *)
let cmpTs: T.testT
=
	let chk (name, n1, n2, expected) = fun state ->
		let s = sign (Num.Z.cmp n1 n2) in
		if s = expected then
			T.succeed state
		else
			T.fail name (pr s) state
	in
	let tcs = [
		"zz0", Num.Z.z, Num.Z.z, Z;
		"zu0", Num.Z.z, Num.Z.u, N;
		"uz0", Num.Z.u, Num.Z.z, P;
		"pz0", Num.Z.mk 2, Num.Z.z, P;
		"zp0", Num.Z.z, Num.Z.mk 2, N;
		"nz0", Num.Z.mk (-2), Num.Z.z, N;
		"zn0", Num.Z.z, Num.Z.mk (-2), P
	] in
	T.suite "cmp" (List.map chk tcs)

(* Num.Z.neg *)
let negTs: T.testT
=
	let chk (name, n, expected) = fun state ->
		let negN = Num.Z.neg n in
		if Num.Z.cmp negN expected = 0 then
			T.succeed state
		else
			T.fail name (Num.Z.pr negN) state
	in
	let tcs = [
		"z0", Num.Z.z, Num.Z.z;
		"u0", Num.Z.u, Num.Z.mk (-1);
		"u1", Num.Z.mk (-1), Num.Z.u;
		"p0", Num.Z.mk 2, Num.Z.mk (-2);
		"n0", Num.Z.mk (-2), Num.Z.mk 2
	] in
	T.suite "neg" (List.map chk tcs)

(* Num.Z.lAnd *)
let lAndTs: T.testT
=
	let chkRes (name, n1, n2, expected) = fun state ->
		let r = Num.Z.lAnd n1 n2 in
		if Num.Z.cmp r expected = 0 then
			T.succeed state
		else
			T.fail name (Num.Z.pr r) state
	in
	let chkSym (name, n1, n2, _) = fun state ->
		let r1 = Num.Z.lAnd n1 n2 in
		let r2 = Num.Z.lAnd n2 n1 in
		if Num.Z.cmp r1 r2 = 0 then
			T.succeed state
		else
			let err = (Num.Z.pr r1) ^ " vs. " ^ (Num.Z.pr r2) in
			T.fail name err state
	in
	let tcs = [
		"z0", Num.Z.z, Num.Z.z, Num.Z.z;
		"z1", Num.Z.u, Num.Z.z, Num.Z.z;
		"z2", Num.Z.mk (-2), Num.Z.z, Num.Z.z;
		"u0", Num.Z.mk 2, Num.Z.u, Num.Z.z;
		"u1", Num.Z.mk 3, Num.Z.u, Num.Z.u;
		"u2", Num.Z.u, Num.Z.u, Num.Z.u;
	] in
	T.suite "lAnd" [
		T.suite "res" (List.map chkRes tcs);
		T.suite "sym" (List.map chkSym tcs)
	]

(* Num.Z.shiftR *)
let shiftRTs: T.testT =
	let chk (name, n, nb, expected) = fun state ->
		let r = Num.Z.shiftR n nb in
		if Num.Z.cmp r expected = 0 then
			T.succeed state
		else
			T.fail name (Num.Z.pr r) state
	in
	let tcs = [
		"z0", Num.Z.z, 0, Num.Z.z;
		"z1", Num.Z.z, 1, Num.Z.z;
		"u0", Num.Z.u, 0, Num.Z.u;
		"u1", Num.Z.u, 1, Num.Z.z;
		"u2", Num.Z.u, 2, Num.Z.z;
		"p0", Num.Z.mk 2, 0, Num.Z.mk 2;
		"p1", Num.Z.mk 2, 1, Num.Z.u;
		"p2", Num.Z.mk 2, 2, Num.Z.z;
		"p3", Num.Z.mk 3, 1, Num.Z.u;
		"n0", Num.Z.mk (-1), 1, Num.Z.mk (-1);
		"n1", Num.Z.mk (-2), 1, Num.Z.mk (-1);
		"n2", Num.Z.mk (-3), 1, Num.Z.mk (-2);
	] in
	T.suite "shiftR" (List.map chk tcs)

let ts: T.testT
= T.suite "Num.Z" [cmpTs; negTs; lAndTs; shiftRTs]
